/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.6 $
 * $Date: 2006/07/22 17:11:46 $
 * $Author: sid4u $
 */

#ifndef _BUFFER_H_
#define _BUFFER_H_

/** @defgroup BufferModule Buffer handling 
 * @ingroup LopsterLibs
 */
/*@{*/

typedef const char* (*StringConvertFunc)(const char** buffer, unsigned* len, unsigned* res_len);

typedef struct _buffer_t buffer_t;

/** The buffer object. */
struct _buffer_t {
  unsigned int size;     ///< the size of the data
  unsigned int max;      ///< the allocated size
  unsigned int consumed; ///< the already comsumed bytes
  char* data;            ///< the data pointer
};



buffer_t* buffer_new(unsigned int size);
void buffer_destroy(buffer_t* buffer);

unsigned int buffer_avail(buffer_t* buffer);
unsigned int buffer_space(buffer_t* buffer);
char* buffer_need_space(buffer_t* buffer, unsigned int space);
void buffer_resize(buffer_t* buffer, unsigned int new_size);

char* buffer_read_pos(buffer_t* buffer);
char* buffer_write_pos(buffer_t* buffer);
void buffer_consume(buffer_t* buffer, unsigned int how_many);
void buffer_consume_virt(buffer_t* buffer, unsigned int how_many);
void buffer_clear(buffer_t* buffer);
void buffer_add_string(buffer_t* buffer, const char* str);
void buffer_add_data(buffer_t* buffer, const void* data, unsigned len);
void buffer_add_converted_string(buffer_t* buffer, const char* str,
                                 StringConvertFunc func);

/*@}*/


#endif
