/* GTK - The GIMP Toolkit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
 * file for a list of people on the GTK+ Team.  See the ChangeLog
 * files for a list of changes.  These files are distributed with
 * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
 */

/*
 * Modified by the Markus Lausser <sgop@users.sourceforge.net>
 */

#ifndef __GTK_FANCYTIPS_H__
#define __GTK_FANCYTIPS_H__

#include <gtk/gtkwidget.h>
#include <gtk/gtkwindow.h>
#include <gtk/gtktreeview.h>


G_BEGIN_DECLS

#define GTK_TYPE_FANCYTIPS                 (gtk_fancytips_get_type ())
#define GTK_FANCYTIPS(obj)                 (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_FANCYTIPS, GtkFancyTips))
#define GTK_FANCYTIPS_CLASS(klass)         (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_FANCYTIPS, GtkFancyTipsClass))
#define GTK_IS_FANCYTIPS(obj)              (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_FANCYTIPS))
#define GTK_IS_FANCYTIPS_CLASS(klass)      (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_FANCYTIPS))
#define GTK_FANCYTIPS_GET_CLASS(obj)       (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_FANCYTIPS, GtkFancyTipsClass))


typedef struct _GtkFancyTips GtkFancyTips;
typedef struct _GtkFancyTipsClass GtkFancyTipsClass;

typedef GtkWidget* (*FancyTipsFunc)(void* data, unsigned depth);

typedef enum {
  GTK_FANCY_POS_MOUSE,
  GTK_FANCY_POS_LEFT,
  GTK_FANCY_POS_RIGHT,
} GtkFancyTipsPos;

struct _GtkFancyTips {
  GtkObject parent_instance;

  GtkWidget* tip_window;

  GtkTreeView* tv;
  void* object;
  FancyTipsFunc func;
  unsigned depth;
  gboolean sticky_mode;
  GtkFancyTipsPos pos;
  
  guint   delay : 30;
  guint	  enabled : 1;
  guint   use_sticky_delay : 1;
  gint	  timer_tag;
  GTimeVal last_popdown;
};

struct _GtkFancyTipsClass {
  GtkObjectClass parent_class;

  /* Padding for future expansion */
  void (*_gtk_reserved1) (void);
  void (*_gtk_reserved2) (void);
  void (*_gtk_reserved3) (void);
  void (*_gtk_reserved4) (void);
};

GType		 gtk_fancytips_get_type(void) G_GNUC_CONST;
GtkFancyTips* gtk_fancytips_new(GtkTreeView* tv, FancyTipsFunc func, 
                                GtkFancyTipsPos pos);

void		 gtk_fancytips_enable	   (GtkFancyTips   *fancytips);
void		 gtk_fancytips_disable   (GtkFancyTips   *fancytips);


G_END_DECLS

#endif /* __GTK_FANCYTIPS_H__ */
