/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.7 $
 * $Date: 2006/07/22 17:11:46 $
 * $Author: sid4u $
 */

#ifndef _HTTP_T_
#define _HTTP_T_

#include "tcp_socket.h"
#include "module.h"

G_BEGIN_DECLS

typedef struct _http_t http_t;

#define CONTENT_UNKNOWN  0
#define CONTENT_RAW      1
#define CONTENT_BZIP2    2

#define ST_INPUT_COMPLETE   200
#define ST_HEADER_ERROR     201

typedef struct {
  object_t _object;
  char* host;
  int port;  
  char* doc;
} url_t;

typedef struct {
  char* name;
  char* value;
} http_tag_t;

typedef struct {
  GList* tags;
} http_tags_t;

#define HTTP(arg) ((http_t*)(arg))
struct _http_t {
  tcp_socket_t _socket;
  // request
  char* host;
  int port;
  char* doc;
  char* proxy;
  gint16 proxy_port;

  // accept
  char* header;
  http_tags_t* header_info;
  gboolean chunked;
  unsigned content_length;
  gboolean keep_alive;
  int content_type;	
  BConsumeInputFunc process_header;
  BConsumeInputFunc process_body;
};

module_t* http_module_instance();

void url_destroy(url_t* url);
void url_init(url_t* url, const char* host, const char* doc);
url_t* url_new(const char* host, const char* doc);
gboolean url_compare(url_t* url1, url_t* url2);


void http_destroy(http_t* http);
gboolean http_end(http_t* http, const char* message);
void http_init(http_t* http, const char* host, const int port, const char* doc);
http_t* http_new(const char* host, const int port, const char* doc);
void http_get(http_t* http);
gboolean http_decompress_bz(http_t* http);


void http_tags_init(http_tags_t* tags);
http_tags_t* http_tags_new(void);
const char* http_tags_find(http_tags_t* tags, const char* name);
void http_tags_add(http_tags_t* tags, const char* name, const char* value);
void http_tags_destroy(http_tags_t* tags);
void http_tags_clear(http_tags_t* tags);

G_END_DECLS

#endif
