/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.2 $
 * $Date: 2006/07/22 17:11:46 $
 * $Author: sid4u $
 */

#ifdef OWN_SOCKET_TRHEAD
static thread_running;
static thread_t* t_socket_manager;
static GStaticMutex socket_lock = G_STATIC_MUTEX_INIT;
#endif

static fd_set ReadFds;
static fd_set WriteFds;
static int lastMaxFd = 0;
static int MaxFd = 0;

#ifdef OWN_SOCKET_TRHEAD
static void backend_set_read(socket_t* s) {
  g_static_mutex_lock(&socket_lock);
  FD_SET(s->priv->fd, &ReadFds);
  g_static_mutex_unlock(&socket_lock);
}

static void backend_clear_read(socket_t* s) {
  g_static_mutex_lock(&socket_lock);
  FD_CLR(s->priv->fd, &ReadFds);
  g_static_mutex_unlock(&socket_lock);
}

static void backend_set_write(socket_t* s) {
  g_static_mutex_lock(&socket_lock);
  FD_SET(s->priv->fd, &WriteFds);
  g_static_mutex_unlock(&socket_lock);
}

static void backend_clear_write(socket_t* s) {
  g_static_mutex_lock(&socket_lock);
  FD_CLR(s->priv->fd, &WriteFds);
  g_static_mutex_unlock(&socket_lock);
}


static void backend_add_socket(socket_t* s) {
  g_static_mutex_lock(&socket_lock);
  if (s->priv->fd > MaxFd) 
  {
      lastMaxFd = MaxFd;
      MaxFd = s->priv->fd;
  }
  g_static_mutex_unlock(&socket_lock);
}

static void backend_remove_socket(socket_t* s) {
  g_static_mutex_lock(&socket_lock);
  //MaxFd = lastMaxFd;
  g_static_mutex_unlock(&socket_lock);
}
#else
static void backend_set_read(socket_t* s) {
  FD_SET(s->priv->fd, &ReadFds);
}

static void backend_clear_read(socket_t* s) {
  FD_CLR(s->priv->fd, &ReadFds);
}

static void backend_set_write(socket_t* s) {
  FD_SET(s->priv->fd, &WriteFds);
}

static void backend_clear_write(socket_t* s) {
  FD_CLR(s->priv->fd, &WriteFds);
}


static void backend_add_socket(socket_t* s) {
  if (s->priv->fd > MaxFd) 
  {
      lastMaxFd = MaxFd;
      MaxFd = s->priv->fd;
  }
}

static void backend_remove_socket(socket_t* s) {
}
#endif

// TODO : handle error on select( error_fds ??)
static gboolean poll_wait_and_dispatch(int* ms) {
  fd_set read_fds;
  fd_set write_fds;
  struct timeval to;
  int ready;
  GList *dlist;

#ifdef OWN_SOCKET_TRHEAD
  if (MaxFd == 0) return TRUE;
#endif

  read_fds = ReadFds;
  write_fds = WriteFds;

  to.tv_sec = *ms/1000;
  to.tv_usec = (*ms%1000)*1000;
#ifdef OWN_SOCKET_TRHEAD
  g_static_mutex_lock(&socket_lock);
#endif 
  ready = select(MaxFd+1, &read_fds, &write_fds, 0, &to); 
  if (ready == -1) {
#ifndef OWN_SOCKET_TRHEAD

    g_log("SOCK", G_LOG_LEVEL_WARNING,
        /*  #ifdef OWN_SOCKET_TRHEAD
          "select(2) failed: %d", WSAGetLastError());
          #else */
          "select(2) failed due to %s (%d)", g_strerror(errno), errno);
       /*   #endif */
#endif
#ifdef OWN_SOCKET_TRHEAD
  g_static_mutex_unlock(&socket_lock);
#endif // OWN_SOCKET_TRHEAD 
    return TRUE;
  }

  // now dispatch
  for (dlist = LSocketModule->sockets; dlist; dlist = dlist->next) {
    socket_t* s = dlist->data;
    if (s->priv->fd == -1) continue;
    
    if (FD_ISSET(s->priv->fd, &read_fds)) {
#ifdef OWN_SOCKET_TRHEAD      
      UI_thread_enter();
#endif      
      s->priv->in_func(s);
#ifdef OWN_SOCKET_TRHEAD      
      UI_thread_leave();
#endif
    }
    //FIXME: we need additional checks to catch side effects (see glib backend)
    if (FD_ISSET(s->priv->fd, &write_fds)) {
#ifdef OWN_SOCKET_TRHEAD
      UI_thread_enter();
#endif
      s->priv->out_func(s);
#ifdef OWN_SOCKET_TRHEAD      
      UI_thread_leave();
#endif    
    }
  }
#ifdef OWN_SOCKET_TRHEAD
  g_static_mutex_unlock(&socket_lock);
#endif // OWN_SOCKET_TRHEAD 
  return TRUE;
}

#ifdef OWN_SOCKET_TRHEAD
//static gboolean Run( int* ms ) {
static unsigned Run(thread_t* socket_manager ) {
  static int timeout = 100;

  while (/*thread_running &&*/ !thread_interrupted(t_socket_manager) )
  {
      while (MaxFd == 0)
      {
          if (/*thread_running == FALSE ||*/ thread_interrupted(t_socket_manager)) 
              return THREAD_COMPLETED;
          Sleep(1);
      }
      poll_wait_and_dispatch(&timeout);       
  }
  return THREAD_COMPLETED;
}
#endif

static void backend_init() {
#ifndef OWN_SOCKET_TRHEAD
  static int timeout = 100;
#endif
#ifdef OWN_SOCKET_TRHEAD
  thread_running = TRUE;
#endif
  FD_ZERO(&ReadFds);
  FD_ZERO(&WriteFds);
#ifdef OWN_SOCKET_TRHEAD
  //  _beginthreadex(NULL,0, (LPTHREAD_START_ROUTINE) Run, &timeout,0,&id); 
  t_socket_manager = thread_new(0);
  thread_set_force_interrupt(t_socket_manager, TRUE);
  thread_start(t_socket_manager, (ThreadMainFunc)Run);
#else
  g_idle_add((GSourceFunc)poll_wait_and_dispatch, &timeout);
#endif
}

#ifdef OWN_SOCKET_TRHEAD
static void backend_end() {
  thread_running = FALSE;
}
#endif
