/* Copyright (C) 2005-2007 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.4 $
 * $Date: 2007/04/29 09:27:54 $
 * $Author: sgop $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <netinet/in.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <gtk/gtk.h>

#include "lcore/preferences.h"
#include "lcore/action_item.h"
#include "lcore/l_base.h"
#include "lcore/l_http.h"
#include "lcore/l_i18n.h"
#include "lcore/gtk2_ui/ui_network.h"
#include "lcore/gtk2_ui/ui_support.h"
#include "lcore/gtk2_ui/ui_main.h"
#include "lcore/gtk2_ui/ui_layout.h"
#include "lcore/gtk2_ui/ui_treeview.h"
#include "dc_network.h"
#include "dc.h"

#define UI_GROUP_ALL    0
#define UI_GROUP_BOOK   1
#define UI_GROUP_ONLINE 2

typedef struct {
  char* country;
  GdkPixbuf* flag;
  unsigned hub_cnt;
  GtkTreeIter* combo_iter;
  GtkTreeIter* tree_iter;
  gboolean dirty;
} ui_flag_t;

typedef struct {
  GtkTreeIter iter;
  GtkListStore* store;
} dc_hub_gui_t;

struct {
  char* text;
  ui_flag_t* flag;
  int group;
} Filter = {
  NULL,
  NULL,
  UI_GROUP_ALL
};

static GtkTreeView* HubView = NULL;
//static GtkBox* NetBox = NULL;
//static GtkListStore* ChannelStore;

static GHashTable* FlagTable = NULL;
static GtkEntry* FilterEntry = NULL;
static GtkComboBox* FilterCountry = NULL;
static GtkComboBox* FilterGroup = NULL;
static ui_flag_t* AnyCountry = NULL;
static GtkWidget* FilterClear = NULL;
static int FlagUpdater = 0;
static GList* FlagsPending = NULL;
static GList* Hubs = NULL;

static GtkWidget* RefreshLabel = NULL;
static GtkWidget* RemoveButton = NULL;


static GtkListStore* ui_dc_get_weblist_store()
{
    static GtkListStore* store = NULL;
    if (!store)
    {
        store = gtk_list_store_new
            (2,
             G_TYPE_POINTER,  // data pointer
             G_TYPE_STRING);  // url
    }
    return store;
}

static GtkTreeModel* ui_dc_hubtree_get()
{
    static GtkListStore* HubList = NULL;
    if (!HubList)
    {
        HubList = gtk_list_store_new
            (8,
             G_TYPE_POINTER,  // data pointer
             G_TYPE_STRING,   // name
             G_TYPE_STRING,   // address
             G_TYPE_INT,      // users
             G_TYPE_STRING,   // status
             G_TYPE_BOOLEAN,  // online?
             G_TYPE_OBJECT,   // flag
             G_TYPE_BOOLEAN   // bookmark?
            );
    }
    return GTK_TREE_MODEL(HubList);
}

static GtkListStore* ui_dc_flaglist_get()
{
    static GtkListStore* FlagList = NULL;
    if (!FlagList)
    {
        ui_flag_t* flag;
        
        FlagList = gtk_list_store_new
            (4,
             G_TYPE_POINTER,  // data pointer
             G_TYPE_STRING,   // name
             G_TYPE_OBJECT,   // flag
             G_TYPE_STRING    // count
            );
        gtk_tree_sortable_set_sort_column_id
            (GTK_TREE_SORTABLE(FlagList), 1, GTK_SORT_ASCENDING);
        
        flag = g_malloc0(sizeof(*flag));
        flag->country = g_strdup("00 Any country");
        flag->flag = NULL;
        flag->hub_cnt = 0;
        flag->combo_iter = g_malloc(sizeof(*flag->combo_iter));
        gtk_list_store_append(FlagList, flag->combo_iter);
        gtk_list_store_set(FlagList, flag->combo_iter,
                           0, flag,
                           1, flag->country,
                           -1);
        AnyCountry = flag;
    }
    return FlagList;
}

static gboolean ui_dc_update_flag_list(void)
{
    if (FilterCountry)
    {
        int num = gtk_tree_model_iter_n_children
            (GTK_TREE_MODEL(ui_dc_flaglist_get()), NULL);
        num = (num+17)/18;
        if (num < 1) num = 1;
        else if (num > 4) num = 4;
        gtk_combo_box_set_wrap_width(FilterCountry, num);
    }
    return FALSE;
}

static gboolean on_update_flag()
{
    GtkListStore* store = ui_dc_flaglist_get();
    GList* dlist;
    
    for (dlist = FlagsPending; dlist; dlist = dlist->next)
    {
        ui_flag_t* flag = dlist->data;
        flag->dirty = FALSE;
        if (!flag->combo_iter) continue;
        gtk_list_store_set(store, flag->combo_iter,
                           3, l_printf("(%u)", flag->hub_cnt),
                           -1);
    }
    gtk_list_store_set(store, AnyCountry->combo_iter,
                       3, l_printf("(%u)", AnyCountry->hub_cnt),
                       -1);
    g_list_free(FlagsPending);
    FlagsPending = NULL;
    g_source_remove(FlagUpdater);
    FlagUpdater = 0;
    return FALSE;
}

static void ui_dc_flag_update(ui_flag_t* flag)
{
    if (flag->dirty) return;
    flag->dirty = TRUE;
    if (FlagUpdater) g_source_remove(FlagUpdater);
    FlagsPending = g_list_prepend(FlagsPending, flag);
    FlagUpdater = g_timeout_add(500, (GSourceFunc)on_update_flag, NULL);
}

static GdkPixbuf* ui_dc_flag_get(dc_hub_t* hub)
{
    ui_flag_t* flag;
    
    if (!hub->country || *hub->country == 0) return NULL;
    
    g_assert(FlagTable);
    flag = g_hash_table_lookup(FlagTable, hub->country);
    
    g_assert(flag);
    return flag->flag;
}

static void ui_dc_hub_hide(dc_hub_t* hub);
static void ui_dc_hub_show(dc_hub_t* hub);

static gboolean _hub_filtered(dc_hub_t* hub)
{
    if (Filter.group == UI_GROUP_ONLINE && hub->status != DC_HUB_ONLINE)
    {
        return TRUE;
    }

    if (Filter.group == UI_GROUP_BOOK && !(hub->flags & DC_HUB_BOOKMARK))
    {
        return TRUE;
    }
    
    if (Filter.flag && Filter.flag != AnyCountry &&
        (!hub->country || strcmp(Filter.flag->country, hub->country)))
    {
        return TRUE;
    }
    
    if (Filter.text &&
        (!hub->name || !strcasestr(hub->name, Filter.text)) &&
        (!hub->description || !strcasestr(hub->description, Filter.text)))
    {
        return TRUE;
    }
    return FALSE;
}

static gboolean on_filter_changed_real()
{
    const char* text;
    GtkTreeIter iter;
    ui_flag_t* flag = NULL;
    int group;
    GList* dlist;
    
    text = ui_entry_get_text(FilterEntry);
    if (text && *text == 0) text = NULL;
    
    if (gtk_combo_box_get_active_iter(FilterCountry, &iter))
    {
        GtkListStore* store = ui_dc_flaglist_get();
        gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 0, &flag, -1);
    }
    
    group = gtk_combo_box_get_active(FilterGroup);
    if (group < UI_GROUP_ALL) group = UI_GROUP_ALL;
    
    g_free(Filter.text);
    Filter.text = g_strdup(text);
    Filter.flag = flag;
    Filter.group = group;
    
    if (Filter.text || (Filter.flag && Filter.flag != AnyCountry) ||
        Filter.group != UI_GROUP_ALL)
        gtk_widget_set_sensitive(FilterClear, TRUE);
    else
        gtk_widget_set_sensitive(FilterClear, FALSE);
    
    for (dlist = Hubs; dlist; dlist = dlist->next)
    {
        dc_hub_t* hub = dlist->data;
        if (!_hub_filtered(hub)) ui_dc_hub_show(hub);
        else ui_dc_hub_hide(hub);
    }
    
    return FALSE;
}

static void on_filter_changed()
{
    lopster_call_periodic(500, (CallFunc)on_filter_changed_real, NULL);
}


static void on_filter_clear()
{
    ui_entry_set_text(FilterEntry, "");
    gtk_combo_box_set_active_iter(FilterCountry, AnyCountry->combo_iter);
    gtk_combo_box_set_active(FilterGroup, UI_GROUP_ALL);
}

static void on_weblist_remove(GtkComboBoxEntry* entry)
{
    GtkListStore* store = ui_dc_get_weblist_store();
    GtkTreeIter iter;
    
    if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(entry), &iter)) {
        url_t* url;
        gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 0, &url, -1);
        object_kill(OBJ(url));
    }
}

static void on_weblist_changed(GtkComboBox* combo)
{
    int index = gtk_combo_box_get_active(combo);
    GtkWidget* entry = gtk_bin_get_child(GTK_BIN(combo));
    const char* text = ui_entry_get_text(GTK_ENTRY(entry));
    
    ui_event_label_set_sensitive(RefreshLabel, text && *text);
    gtk_widget_set_sensitive(RemoveButton, index != -1);
}


static void on_hubs_refresh(GtkBin* entry)
{
    dc_weblist_get_list(ui_entry_get_text(GTK_ENTRY(entry->child)));
}


static void ui_dc_create_net_page(void)
{
    GtkWidget* hbox;
    GtkWidget* vbox;
    GtkWidget* item;
    GtkWidget* label;
    GtkWidget* entry;
    GtkWidget* frame;
    GtkWidget* button;
    GtkWidget* sw;
    GtkWidget* tv;
    GtkTreeViewColumn* column;
    GtkCellRenderer* rend;
    GtkListStore* store;
    
    vbox = gtk_vbox_new(FALSE, 5);
    gtk_widget_show(vbox);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
    
    // Filter
    hbox = gtk_hbox_new(FALSE, 5);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
    
    item = gtk_label_new(l_printf("<b>%s</b>", _("Filter")));
    gtk_label_set_use_markup(GTK_LABEL(item), TRUE);
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    
    entry = ui_entry_new(NULL);
    gtk_widget_show(entry);
    gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 0);
    FilterEntry = GTK_ENTRY(entry);
    g_signal_connect(G_OBJECT(entry), "changed",
                     G_CALLBACK(on_filter_changed), NULL);
    
    item = gtk_combo_box_new_with_model(GTK_TREE_MODEL(ui_dc_flaglist_get()));
    rend = gtk_cell_renderer_pixbuf_new();
    g_object_set(G_OBJECT(rend), "xalign", 1.0, "width", 40, NULL);
    gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(item), rend, FALSE);
    gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(item), rend,
                                   "pixbuf", 2, NULL);
    rend = gtk_cell_renderer_text_new();
    g_object_set(G_OBJECT(rend), "xpad", 5, NULL);
    gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(item), rend, TRUE);
    gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(item), rend,
                                   "text", 1, NULL);
    rend = gtk_cell_renderer_text_new();
    gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(item), rend, FALSE);
    gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(item), rend,
                                   "text", 3, NULL);
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    
    FilterCountry = GTK_COMBO_BOX(item);
    g_signal_connect(G_OBJECT(item), "changed",
                     G_CALLBACK(on_filter_changed), NULL);
    ui_dc_update_flag_list();
    
    
    item = gtk_combo_box_new_text();
    gtk_combo_box_append_text(GTK_COMBO_BOX(item), _("Show all hubs"));
    gtk_combo_box_append_text(GTK_COMBO_BOX(item), _("Show bookmarks"));
    gtk_combo_box_append_text(GTK_COMBO_BOX(item), _("Show online hubs"));
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    FilterGroup = GTK_COMBO_BOX(item);
    g_signal_connect(G_OBJECT(item), "changed",
                     G_CALLBACK(on_filter_changed), NULL);
    
    FilterClear = item = ui_create_close_button(_("clear filter"));
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    gtk_widget_set_sensitive(FilterClear, FALSE);
    g_signal_connect_swapped(G_OBJECT(FilterClear), "clicked",
                             G_CALLBACK(on_filter_clear), NULL);
    
    frame = gtk_frame_new(NULL);
    
    item = ui_create_show_hide_label(_("web hublist"), _("collapse"), frame);
    gtk_widget_show(item);
    gtk_box_pack_end(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    
    item = ui_create_event_label
        (_("add hub"), G_CALLBACK(dc_hub_edit), NULL, TRUE);
    gtk_widget_show(item);
    gtk_box_pack_end(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    
    // Web location
    gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
    
    hbox = gtk_hbox_new(FALSE, 5);
    gtk_widget_show(hbox);
    gtk_container_add(GTK_CONTAINER(frame), hbox);
    gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
    
    item = gtk_label_new(_("hublist web location"));
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    
    RemoveButton = button = ui_create_close_button(_("remove this web location"));
    gtk_widget_show(button);
    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
    
    store = ui_dc_get_weblist_store();
    entry = gtk_combo_box_entry_new_with_model(GTK_TREE_MODEL(store), 1);
    gtk_widget_show(entry);
    gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);

    g_signal_connect_swapped(G_OBJECT(RemoveButton), "clicked",
                             G_CALLBACK(on_weblist_remove), entry);

    RefreshLabel = label = ui_create_event_label
        (_("get hub list"), G_CALLBACK(on_hubs_refresh), entry, TRUE);
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

    if (gtk_combo_box_get_active(GTK_COMBO_BOX(entry)) == -1)
    {
        gtk_widget_set_sensitive(RemoveButton, FALSE);
        ui_event_label_set_sensitive(RefreshLabel, FALSE);
    }

    g_signal_connect(G_OBJECT(entry), "changed",
                     G_CALLBACK(on_weblist_changed), NULL);
  
    // HUBLIST
    sw = gtk_scrolled_window_new(NULL, NULL);
    gtk_widget_show(sw);
    gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), 
                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW (sw),
                                        GTK_SHADOW_IN);

    tv = gtk_tree_view_new_with_model(ui_dc_hubtree_get());
    //  gtk_tree_view_columns_autosize(GTK_TREE_VIEW(tv));
    gtk_container_add(GTK_CONTAINER(sw), tv);
    gtk_widget_show(tv);
    gtk_tree_view_set_headers_clickable (GTK_TREE_VIEW (tv), TRUE);
    g_object_set(G_OBJECT(tv), "fixed-height-mode", TRUE, NULL);
    //  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tv), TRUE);
    HubView = GTK_TREE_VIEW(tv);

    gtk_tree_selection_set_mode
        (gtk_tree_view_get_selection(GTK_TREE_VIEW (tv)),
         GTK_SELECTION_MULTIPLE);

    rend = gtk_cell_renderer_pixbuf_new();
    g_object_set(G_OBJECT(rend), "xalign", 0.0, "xpad", 5, NULL);
    column = ui_tv_column_new
        (_("Name"),
         rend, "pixbuf", 6, FALSE,
         gtk_cell_renderer_text_new(), "markup", 1, TRUE,
         NULL);
    ui_tv_config_default_col(column, 1, "dc:hub:col:name", FALSE);
    gtk_tree_view_append_column(GTK_TREE_VIEW (tv), column);

    column = ui_tv_column_new
        (_("Status"), gtk_cell_renderer_text_new(), "text", 4, TRUE, NULL);
    ui_tv_config_default_col(column, 4, "dc:hub:col:status", FALSE);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tv), column);

    column = ui_tv_column_new
        (_("Users"), gtk_cell_renderer_text_new(), "text", 3, TRUE, NULL);
    ui_tv_config_default_col(column, 3, "dc:hub:col:users", FALSE);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tv), column);

    {
        action_item_t* actions = action_item_search("DirectConnect hub actions");
        ui_tv_handle_button(GTK_TREE_VIEW(tv), 1, &actions, NULL);
    }


    ui_add_network_subpage(DC_ID, vbox);

    //  ui_tv_config_line_marker(GTK_TREE_VIEW(tv), "network:entry:online", 6);
}

void ui_dc_net_init()
{
    ui_dc_create_net_page();
}

static void ui_dc_hub_hide(dc_hub_t* hub)
{
    dc_hub_gui_t* data = object_get_data(OBJ(hub), "HubEntry");

    if (!data) return;
    
    gtk_list_store_remove(data->store, &(data->iter));
    object_free_data_common(OBJ(hub), "HubEntry");
}

static void ui_dc_hub_update(dc_hub_t* hub)
{
    dc_hub_gui_t* data = object_get_data(OBJ(hub), "HubEntry");
    char* text;
    
    if (!data) return;
    
    text = g_markup_printf_escaped
        ("%s\n<span size=\"smaller\" foreground=\"#99aa99\">%s</span>", 
         hub->name, hub->description);
    gtk_list_store_set(data->store, &data->iter,
                       1, text,
                       2, l_printf("%s:%d", hub->address, ntohs(TSOCK(hub)->port)),
                       3, dc_hub_get_user_cnt(hub),
                       7, (hub->flags & DC_HUB_BOOKMARK),
                       -1);
    g_free(text);

    if (hub->status == DC_HUB_ONLINE)
    {
        gtk_list_store_set(data->store, &data->iter,
                           4, _("Online"),
                           -1);
    }
}

static void ui_dc_hub_show(dc_hub_t* hub)
{
    dc_hub_gui_t* data;
    GtkListStore* store;
    
    if (!hub) return;
    if (object_get_data(OBJ(hub), "HubEntry")) return;;
    
    store = GTK_LIST_STORE(ui_dc_hubtree_get());
    
    object_set_data(OBJ(hub), "HubEntry", data = g_malloc0(sizeof(*data)));
    data->store = store;
    gtk_list_store_append(data->store, &data->iter);
    gtk_list_store_set(data->store, &data->iter,
                       0, hub,
                       6, ui_dc_flag_get(hub),
                       -1);
    
    ui_dc_hub_update(hub);
}

static void ui_dc_hub_unregister_flag(dc_hub_t* hub)
{
    ui_flag_t* flag;
    
    AnyCountry->hub_cnt--;
    Hubs = g_list_remove(Hubs, hub);
    
    if (!hub->country || *hub->country == 0) return;
    
    flag = g_hash_table_lookup(FlagTable, hub->country);
    flag->hub_cnt--;
    if (flag->hub_cnt == 0)
    {
        GtkListStore* store = ui_dc_flaglist_get();
        gtk_list_store_remove(store, flag->combo_iter);
        g_free(flag->combo_iter);
        flag->combo_iter = NULL;
        lopster_call_periodic(2000, (CallFunc)ui_dc_update_flag_list, NULL);
    }
    ui_dc_flag_update(flag);
}

static void ui_dc_hub_destroy(dc_hub_t* hub)
{
    ui_dc_hub_unregister_flag(hub);
    ui_dc_hub_hide(hub);
}

static void ui_dc_flag_destroy(ui_flag_t* flag)
{
    g_free(flag->country);
    if (flag->flag) gdk_pixbuf_unref(flag->flag);
    g_free(flag->combo_iter);
    g_free(flag);
}

static void ui_dc_hub_register_flag(dc_hub_t* hub)
{
    ui_flag_t* flag;
    
    AnyCountry->hub_cnt++;
    Hubs = g_list_prepend(Hubs, hub);
    
    if (!hub->country || *hub->country == 0) return;
    
    if (!FlagTable)
    {
        FlagTable = g_hash_table_new_full
            (g_str_hash, g_str_equal, NULL,(GDestroyNotify)ui_dc_flag_destroy);
    }
  
    flag = g_hash_table_lookup(FlagTable, hub->country);
    if (!flag)
    {
        // create new flag
        flag = g_malloc0(sizeof(*flag));
        flag->country = g_strdup(hub->country);
        flag->flag = create_pixbuf(l_printf("Flag%s.png", hub->country));
        if (!flag->flag) flag->flag = create_pixbuf("FlagUnknown.png");
        flag->hub_cnt = 0;
        
        // insert in hash table
        g_hash_table_insert(FlagTable, flag->country, flag);
    }
    
    flag->hub_cnt++;
    if (!flag->combo_iter)
    {
        GtkListStore* store = ui_dc_flaglist_get();
        // insert in ComboBox
        flag->combo_iter = g_malloc(sizeof(*flag->combo_iter));
        gtk_list_store_append(store, flag->combo_iter);
        gtk_list_store_set(store, flag->combo_iter,
                           0, flag,
                           1, flag->country,
                           2, flag->flag,
                           -1);
        lopster_call_periodic(2000, (CallFunc)ui_dc_update_flag_list, NULL);
    }
    ui_dc_flag_update(flag);
}

void ui_dc_hub_new(dc_hub_t* hub)
{
    g_assert(hub);

    if (object_get_data(OBJ(hub), "HubEntry")) return;;
    
    ui_dc_hub_register_flag(hub);
    
    object_notify_d(OBJ(hub), (ObjectFunc)ui_dc_hub_destroy, NULL);
    object_monitor_d(OBJ(hub), (ObjectFunc)ui_dc_hub_update, NULL);
    
    if (!_hub_filtered(hub)) ui_dc_hub_show(hub);
}

void ui_dc_hub_set_status(dc_hub_t* hub, const char* message)
{
    dc_hub_gui_t* data = object_get_data(OBJ(hub), "HubEntry");
    
    if (!data) return;

    gtk_list_store_set(data->store, &data->iter,
                       4, message,
                       5, (hub->status == DC_HUB_ONLINE),
                       -1);
}

static void ui_dc_weblist_hide(url_t* web)
{
    GtkTreeIter* iter = object_get_data(OBJ(web), "HubListUrl");
    
    if (!iter) return;

    gtk_list_store_remove(ui_dc_get_weblist_store(), iter);
    object_free_data_common(OBJ(web), "HubListUrl");
}

static void ui_dc_weblist_update(url_t* web)
{
    GtkTreeIter* iter = object_get_data(OBJ(web), "HubListUrl");
    
    if (!iter) return;
    
    gtk_list_store_set(ui_dc_get_weblist_store(), iter,
                       0, web,
                       1, l_printf("http://%s%s", web->host, web->doc),
                       -1);
}

static void ui_dc_weblist_show(url_t* web)
{
    GtkTreeIter* iter;
    GtkListStore* store = ui_dc_get_weblist_store();
    
    if (object_get_data(OBJ(web), "HubListUrl")) return;

    object_set_data(OBJ(web), "HubListUrl", iter = g_malloc(sizeof(*iter)));
    gtk_list_store_append(store, iter);
    ui_dc_weblist_update(web);
}


void ui_dc_weblist_new(url_t* web)
{
    g_assert(web);
    
    object_notify_d(OBJ(web), (ObjectFunc)ui_dc_weblist_hide, NULL);
    object_monitor_d(OBJ(web), (ObjectFunc)ui_dc_weblist_update, NULL);
    
    ui_dc_weblist_show(web);
}

static void on_hub_edit_clicked(GObject* button, dc_hub_t* hub)
{
    GtkWidget* win;
    const char* nick;
    const char* pass;
    const char* name;
    const char* des;
    const char* address;
    const char* charset;
    gboolean new_hub = FALSE;
    port_t port;
    
    nick = ui_entry_get_text(g_object_get_data(button, "nick"));
    if (strchr(nick, ' '))
    {
        lopster_notify(LOPSTER_MESSAGE_ERROR, _("Invalid input"),
                       _("The username may not contain any spaces"));
        return;
    }
    
    pass = ui_entry_get_text(g_object_get_data(button, "pass"));
    if (strchr(pass, ' '))
    {
        lopster_notify(LOPSTER_MESSAGE_ERROR, _("Invalid input"),
                       _("The password may not contain any spaces"));
        return;
    }
    
    name = ui_entry_get_text(g_object_get_data(button, "name"));
    if (*name == 0)
    {
        lopster_notify(LOPSTER_MESSAGE_ERROR, _("Invalid input"),
                       _("The hub name is empty"));
        return;
    }
    
    address = ui_entry_get_text(g_object_get_data(button, "address"));
    if (*address == 0)
    {
        lopster_notify(LOPSTER_MESSAGE_ERROR, _("Invalid input"),
                       _("The hub address is empty"));
        return;
    }
    
    port = gtk_spin_button_get_value(g_object_get_data(button, "port"));
    
    if (!hub)
    {
        hub = dc_hub_new(name, address, port);
        new_hub = TRUE;
    }
    else
    {
        g_free(hub->address);
        hub->address = g_strdup(address);
        TSOCK(hub)->port = htons(port);
    }
    
    if (gtk_toggle_button_get_active(g_object_get_data(button, "book")))
        hub->flags |= DC_HUB_BOOKMARK;
    else
        hub->flags &= ~DC_HUB_BOOKMARK;
    
    des = ui_entry_get_text(g_object_get_data(button, "description"));
    g_free(hub->description);
    hub->description = g_strdup(des);
    
    g_free(hub->h_nick);
    hub->h_nick = g_strdup(nick);
    
    g_free(hub->h_pass);
    hub->h_pass = g_strdup(pass);
    
    charset = ui_charset_control_get_charset(g_object_get_data(button, "charset"));
    g_free(hub->charset);
    hub->charset = g_strdup(charset);
  
/*   hub->country = "unknown"; */
  
    if (new_hub)
    {
        GtkWidget* entry = gtk_bin_get_child(g_object_get_data(button, "country"));
        hub->country = g_strdup(ui_entry_get_text(GTK_ENTRY(entry)));
        ui_dc_hub_new(hub);
    }
    else
    {
        dc_hub_set_name(hub, name);
    }

    win = g_object_get_data(button, "win");
    gtk_widget_destroy(win);
}

void ui_dc_hub_edit(dc_hub_t* hub)
{
    GtkWidget* win;
    GtkWidget* vbox;
    GtkWidget* hbox;
    GtkWidget* hbox2;
    GtkWidget* entry;
    GtkWidget* button;
    GtkWidget* item;
    GtkWidget* spin;
    GtkWidget* vbox2;
    
    win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    if (hub)
        gtk_window_set_title(GTK_WINDOW(win), _("Edit hub"));
    else
        gtk_window_set_title(GTK_WINDOW(win), _("Create new hub"));
    button = gtk_button_new_from_stock(GTK_STOCK_OK);
    gtk_window_set_type_hint(GTK_WINDOW(win), GDK_WINDOW_TYPE_HINT_DIALOG);
    gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_CENTER_ON_PARENT);
    if (ui_main_get_win())
    {
        gtk_window_set_transient_for(GTK_WINDOW(win), ui_main_get_win());
        gtk_window_set_destroy_with_parent(GTK_WINDOW(win), TRUE);
    }
    
    vbox = gtk_vbox_new(FALSE, 10);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
    gtk_container_add(GTK_CONTAINER(win), vbox);

    item = ui_create_section(_("Hub settings"), NULL, &hbox, NULL);
    gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);
    
    vbox2 = gtk_vbox_new(FALSE, 10);
    gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, TRUE, 0);
    
    //
    hbox = gtk_hbox_new(FALSE, 5);
    
    entry = ui_entry_new_default(hub?hub->name:NULL, _("Enter name here"));
    g_object_set_data(G_OBJECT(button), "name", entry);
    gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
    
    entry = gtk_check_button_new_with_label(_("bookmarked"));
    if (hub && (hub->flags & DC_HUB_BOOKMARK))
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(entry), TRUE);
    g_object_set_data(G_OBJECT(button), "book", entry);
    gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 0);
    
    item = ui_create_headed_item(_("hub name"), hbox);
    gtk_box_pack_start(GTK_BOX(vbox2), item, FALSE, FALSE, 0);
    
    //
    entry = ui_entry_new(hub && hub->description?hub->description:NULL);
    g_object_set_data(G_OBJECT(button), "description", entry);
    
    item = ui_create_headed_item(_("description"), entry);
    gtk_box_pack_start(GTK_BOX(vbox2), item, FALSE, FALSE, 0);
    
    //
    hbox = gtk_hbox_new(FALSE, 5);
    gtk_box_pack_start(GTK_BOX(vbox2), hbox, FALSE, FALSE, 0);
    
    entry = ui_entry_new_default(hub?hub->address:NULL, _("Enter address here"));
    g_object_set_data(G_OBJECT(button), "address", entry);
    
    item = ui_create_headed_item(_("address"), entry);
    gtk_box_pack_start(GTK_BOX(hbox), item, TRUE, TRUE, 0);
    
    spin = gtk_spin_button_new_with_range(0, 65535, 1);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), hub?ntohs(TSOCK(hub)->port):411);
    gtk_spin_button_set_increments(GTK_SPIN_BUTTON(spin), 1, 100);
    g_object_set_data(G_OBJECT(button), "port", spin);
    
    item = ui_create_headed_item(_("port"), spin);
    gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    
    //
    hbox = gtk_hbox_new(FALSE, 10);
    gtk_box_pack_start(GTK_BOX(vbox2), hbox, FALSE, FALSE, 0);
    
    entry = gtk_combo_box_entry_new_with_model(GTK_TREE_MODEL(ui_dc_flaglist_get()), 1);
    {
        GtkCellRenderer* rend = gtk_cell_renderer_pixbuf_new();
        
        g_object_set(G_OBJECT(rend), "xalign", 1.0, "width", 40, NULL);
        gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(entry), rend, FALSE);
        gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(entry), rend,
                                       "pixbuf", 2, NULL);
        gtk_cell_layout_reorder(GTK_CELL_LAYOUT(entry), rend, 0);
    }
    {
        int num = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(ui_dc_flaglist_get()), NULL);
        
        num = (num+17)/18;
        if (num < 1) num = 1;
        else if (num > 4) num = 4;
        gtk_combo_box_set_wrap_width(GTK_COMBO_BOX(entry), num);
/*     gtk_combo_box_set_wrap_width(GTK_COMBO_BOX(entry), 2); */
    }
    g_object_set_data(G_OBJECT(button), "country", entry);
    if (hub && hub->country)
        ui_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry))), hub->country);
    
    item = ui_create_headed_item(_("country"), entry);
    gtk_box_pack_start(GTK_BOX(hbox), item, TRUE, TRUE, 0);
    if (hub) gtk_widget_set_sensitive(item, FALSE); //FIXME: allow to set/change country
    
    entry = ui_create_charset_control(hub?hub->charset:NULL, NULL, NULL);
    g_object_set_data(G_OBJECT(button), "charset", entry);
    
    item = ui_create_headed_item(_("encoding"), entry);
    gtk_box_pack_start(GTK_BOX(hbox), item, TRUE, TRUE, 0);
    
    
    
    item = ui_create_section
        (_("Override login"),
         _("Only enter a username or password here if you like to use other "
           "settings than the default for this hub."),
         &hbox, NULL);
    gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);
    
    
    hbox2 = gtk_hbox_new(TRUE, 10);
    gtk_box_pack_start(GTK_BOX(hbox), hbox2, TRUE, TRUE, 0);
    
    //
    entry = ui_entry_new_default(hub && hub->h_nick?hub->h_nick:NULL, _("Using default username"));
    g_object_set_data(G_OBJECT(button), "nick", entry);
    
    item = ui_create_headed_item(_("username"), entry);
    gtk_box_pack_start(GTK_BOX(hbox2), item, TRUE, TRUE, 0);
    
    entry = ui_entry_new_default_pass(hub && hub->h_pass?hub->h_pass:NULL, _("Using default password"));
    g_object_set_data(G_OBJECT(button), "pass", entry);
    
    item = ui_create_headed_item(_("password"), entry);
    gtk_box_pack_start(GTK_BOX(hbox2), item, TRUE, TRUE, 0);
    
    ////////////////
    hbox = ui_create_bbox(10, GTK_BUTTONBOX_END);
    gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

    {
        GtkWidget* button2 = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
        gtk_container_add(GTK_CONTAINER(hbox), button2);
        g_signal_connect_swapped(G_OBJECT(button2), "clicked",
                                 G_CALLBACK(gtk_widget_destroy), win);
    }
    
    gtk_container_add(GTK_CONTAINER(hbox), button);
    g_signal_connect(G_OBJECT(button), "clicked",
                     G_CALLBACK(on_hub_edit_clicked), hub);
    g_object_set_data(G_OBJECT(button), "win", win);
    
    gtk_widget_show_all(win);
}
