/* Copyright (C) 2005-2007 sgop@users.sourceforge.net
 * This is free software distributed under the terms of
 * the GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.14 $
 * $Date: 2007/04/29 09:28:37 $
 * $Author: sgop $
 */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <gtk/gtk.h>

#include "lcore/utils.h"
#include "lcore/buffer.h"
#include "lcore/preferences.h"
#include "lcore/l_base.h"
#include "lcore/l_i18n.h"
#include "lcore/xml_handler.h"
#include "lcore/gtk2_ui/ui_network.h"
#include "lcore/gtk2_ui/ui_layout.h"
#include "lcore/gtk2_ui/ui_support.h"
#include "lcore/gtk2_ui/ui_main.h"
#include "lcore/gtk2_ui/ui_style.h"
#include "lcore/gtk2_ui/ui_treeview.h"
#include "lcore/gtk2_ui/ui_init.h"
#include "lcore/gtk2_ui/ui_tooltips.h"
#include "nap_network.h"
#include "nap_napi.h"
#include "nap_chat.h"
#include "nap.h"

#include "pixmaps/server_perm.xpm"
#include "pixmaps/server_deact.xpm"
#include "pixmaps/server_napi.xpm"
#include "pixmaps/server_nnapi.xpm"

static GtkTreeView* ServerView = NULL;

static GdkPixbuf* ServerIgn = NULL;
static GdkPixbuf* ServerPerm = NULL;
static GdkPixbuf* ServerNapi = NULL;
static GdkPixbuf* ServerNNapi = NULL;

static GtkWidget* RefreshLabel = NULL;
static GtkWidget* RemoveButton = NULL;

static void ui_nap_create_net_page(void);
static void ui_nap_ng_show(nap_ng_t* ng);
static void ui_nap_net_show(nap_net_t* net);
static void ui_nap_server_show(nap_server_t* server);

typedef struct {
  GtkToggleButton* timport;
  GtkToggleButton* tnothing;
  gboolean done;
} ui_net_t;

static GtkWidget* ui_net_create(ui_page_t* page) {
  GtkWidget* vbox;
  GtkWidget* toggle;
  ui_net_t* data = g_malloc0(sizeof(*data));

  ui_page_set_data(page, data);

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(vbox);

  toggle = gtk_radio_button_new_with_label
    (NULL, _("Import networks from Lopster 1"));
  data->timport = GTK_TOGGLE_BUTTON(toggle);
  gtk_widget_show(toggle);
  gtk_box_pack_start(GTK_BOX(vbox), toggle, FALSE, FALSE, 0);

  toggle = gtk_radio_button_new_with_label
    (gtk_radio_button_get_group(GTK_RADIO_BUTTON(toggle)), _("Get a fresh list later"));
  data->tnothing = GTK_TOGGLE_BUTTON(toggle);
  gtk_widget_show(toggle);
  gtk_box_pack_start(GTK_BOX(vbox), toggle, FALSE, FALSE, 0);

  gtk_toggle_button_set_active(data->timport, TRUE);
  return vbox;
}

static const char* ui_net_enter(ui_page_t* page, gboolean next) {
  ui_net_t* data = ui_page_get_data(page);
  (void)next;
  
  if (data->done) {
    gtk_widget_set_sensitive(GTK_WIDGET(data->timport), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(data->tnothing), FALSE);
    return _("Networks were already imported");
  }
  return NULL;
}

static int ui_ng_flags_from_lopster1(int flags) {
  int res = 0;
  if (!(flags & 0x4)) res |= NAP_NG_AUTO_CONNECT;
  return res;
}
static int ui_net_flags_from_lopster1(int flags) {
  int res = 0;
  if (flags & 0x2) res |= NAP_NET_NO_WINMX;
  if (flags & 0x4) res |= NAP_NET_OWN_SERVER;
  if (flags & 0x8) res |= NAP_NET_IGN_GLOBAL;
  return res;
}
static int ui_server_flags_from_lopster1(int flags) {
  int res = 0;
  if (flags & 0x4) res |= NAP_SERVER_IGNORE;
  if (!(flags & 0x8)) res |= NAP_SERVER_NAPI;
  // we currently don't support meta servers (flag 0x10)
  return res;
}

static const char* ui_net_import(FILE* file) {
  char line[2048];
  nap_ng_t* ng = NULL;
  nap_net_t* net = NULL;
  nap_server_t* server;
  gboolean skip_server = FALSE;

  while (mfgets(line, sizeof(line), file)) {
    char* name = arg(line, 0);
    if (!name) continue;
    if (*name == '+') {
      char* ng_name = arg(NULL, 2);
      char* flags = arg(NULL, 0);
      char* keep = arg(NULL, 0);
      int temp;

      if (!flags) continue;
      temp = ui_ng_flags_from_lopster1(atoi(flags));

      ng = nap_ng_create(ng_name, temp, NAP_CREATE_REPLACE);
      if (keep) ng->keep = atoi(keep);
      else if (atoi(flags) & 0x1) ng->keep = -1; // backward
    } else if (*name == '-') {
      char* address = arg(NULL, 2);
      char* ip = arg(NULL, 2);
      char* port = arg(NULL, 2);
      char* flags = arg(NULL, 2);

      if (!net || skip_server) continue;
      if (!flags) continue;

      server = nap_server_create(net, address, NAP_CREATE_UNIQUE);
      if (server) {
        server->ip = strtoul(ip, NULL, 10);
        server->port = atoi(port);
        server->flags = ui_server_flags_from_lopster1(atoi(flags));
        // move one flag from server to net
        if (atoi(flags) & 0x2) net->flags |= NAP_NET_PERSISTENT;
      }
    } else {
      char* flags = arg(NULL, 2);
      char* join = arg(NULL, 2);
      char* nick = arg(NULL, 2);
      char* pass = arg(NULL, 2);
      char* type = arg(NULL, 2);
      char* codeset = arg(NULL, 2);
      //      char* max_share = arg(NULL, 2);   //unused
      char* nnick = nap_net_get_nick(NULL);
      char* npass = nap_net_get_pass(NULL);

      if (!type || atoi(type) != 1 || !ng) {
        skip_server = TRUE;
        continue;
      }
      skip_server = FALSE;

      net = nap_net_create(ng, name, NAP_CREATE_UNIQUE);
      if (net) {
        net->flags = ui_net_flags_from_lopster1(atoi(flags));
        if (!nnick || strcmp(nnick, nick))
          nap_net_set_nick(net, nick);
        if (!npass || strcmp(npass, pass))
          nap_net_set_password(net, pass);
        if (codeset && *codeset)
          net->charset = g_strdup(codeset);

        make_list_from_string(&net->autojoin, join, " ");
      }
    }
  }
  return NULL;
}

static const char* ui_net_leave(ui_page_t* page, gboolean next) {
  ui_net_t* data = ui_page_get_data(page);

  if (!next) return NULL;
  if (data->done) return NULL;

  if (gtk_toggle_button_get_active(data->timport)) {
    char* filename = l_printf
      (SDIR("%s") SDIR(".lopster") "server.list", g_get_home_dir());
    FILE* file;
    const char* res;

    file = fopen(filename, "r");
    if (!file)
      return _("Failed to import networks from Lopster 1");
    
    res = ui_net_import(file);
    fclose(file);
    if (res) return res;

    data->done = TRUE;
  }
  return NULL;
}

void ui_nap_net_init() {
  ServerIgn   = gdk_pixbuf_new_from_xpm_data(server_deact_xpm);
  ServerPerm  = gdk_pixbuf_new_from_xpm_data(server_perm_xpm);
  ServerNapi  = gdk_pixbuf_new_from_xpm_data(server_napi_xpm);
  ServerNNapi = gdk_pixbuf_new_from_xpm_data(server_nnapi_xpm);

  ui_nap_create_net_page();

  if (!lopster_xml_loaded("nap:serverlist")) {
    char* filename = l_printf
      (SDIR("%s") SDIR(".lopster") "server.list", g_get_home_dir());
    if (g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
      ui_init_add(_("Napster network list"),
                  _("You can import the network list from Lopster 1. Please note, "
                    "that Lopster 2 handles the login/password not the same way Lopster 1 does. "
                    "In Lopster 2 networks usually don't have their own login/password "
                    "specified, instead the main login/password is used. "
                    "If you import the networks from Lopster 1, then all logins/passwords "
                    "will be stripped that appear to be same as the global login/password."),
                  ui_net_create, ui_net_enter, ui_net_leave);
    }
  }
}

static GtkTreeStore* ui_nap_get_net_tree() {
  static GtkTreeStore *ServerTree = NULL;
  if (!ServerTree) {
    ServerTree = gtk_tree_store_new
      (11,
       G_TYPE_POINTER,  // data pointer
       G_TYPE_STRING,   // name
       G_TYPE_BOOLEAN,  // online?
       G_TYPE_STRING,   // status
       G_TYPE_INT,      // 
       G_TYPE_INT,
       G_TYPE_INT,
       G_TYPE_OBJECT,   // napi/persistent pix
       G_TYPE_OBJECT,   // deact pix
       G_TYPE_INT,
       G_TYPE_STRING);
  }
  return ServerTree;
}

static GtkListStore* ui_nap_get_service_store() {
  static GtkListStore* store = NULL;
  if (!store) {
    store = gtk_list_store_new
      (2,
       G_TYPE_POINTER,  // data pointer
       G_TYPE_STRING);  // url
  }
  return store;
}

static void ui_nap_server_update(nap_server_t* server);
static void ui_nap_server_hide(nap_server_t* server)
{
    GtkTreeIter* node = object_get_data(OBJ(server), "NapServerEntry");

    if (!node) return;

    object_unmonitor_d(OBJ(server), (ObjectFunc)ui_nap_server_update, NULL);
    
    gtk_tree_store_remove(ui_nap_get_net_tree(), node);
    object_free_data_common(OBJ(server), "NapServerEntry");
}

static void ui_nap_net_update(nap_net_t* net);
static void ui_nap_net_hide(nap_net_t* net)
{
    GtkTreeIter* node = object_get_data(OBJ(net), "NapNetEntry");
    GList* dlist;

    if (!node) return;

/*   object_monitor_remove(OBJ(net), (ObjectFunc)ui_nap_net_update, NULL); */
    
    for (dlist = net->servers; dlist; dlist = dlist->next)
    {
        nap_server_t* server = dlist->data;
        ui_nap_server_hide(server);
    }

    gtk_tree_store_remove(ui_nap_get_net_tree(), node);
    object_free_data_common(OBJ(net), "NapNetEntry");
}

static void ui_nap_ng_update(nap_ng_t* ng);
static void ui_nap_ng_hide(nap_ng_t* ng)
{
    GtkTreeIter* node = object_get_data(OBJ(ng), "NapGroupEntry");
    GList* dlist;
    
    if (!node) return;
    
    object_unmonitor_d(OBJ(ng), (ObjectFunc)ui_nap_ng_update, NULL);
    
    for (dlist = ng->nets; dlist; dlist = dlist->next)
    {
        nap_net_t* net = dlist->data;
        ui_nap_net_hide(net);
    }
    
    gtk_tree_store_remove(ui_nap_get_net_tree(), node);
    object_free_data_common(OBJ(ng), "NapGroupEntry");
}

static void ui_nap_server_update(nap_server_t* server)
{
    GtkTreeIter* node = object_get_data(OBJ(server), "NapServerEntry");
    
    // check whether node is shown
    if (!node)
    {
        ui_nap_server_show(server);
        return;
    }
    else if (!server->net)
    {
        ui_nap_server_hide(server);
        return;
    }
    
    gtk_tree_store_set(ui_nap_get_net_tree(), node,
                       1, OBJ(server)->name,  //FIXME
                       2, (server->status == NAP_SERVER_ONLINE),
                       3, server->message,
                       4, server->users,
                       5, server->files,
                       6, server->gigs,
                       7, (server->flags & NAP_SERVER_NAPI)?ServerNapi:ServerNNapi,
                       8, (server->flags & NAP_SERVER_IGNORE)?ServerIgn:NULL,
                       -1);
}

static void ui_nap_server_show(nap_server_t* server)
{
    GtkTreeIter* node = object_get_data(OBJ(server), "NapServerEntry");
    GtkTreeIter* parent;
    
    if (!server || !server->net) return;
    if (node) return;
    
    parent = object_get_data(OBJ(server->net), "NapNetEntry");
    if (!parent) return;

    object_set_data(OBJ(server), "NapServerEntry", node = g_malloc(sizeof(*node)));
    gtk_tree_store_append(ui_nap_get_net_tree(), node, parent);

    gtk_tree_store_set(ui_nap_get_net_tree(), node,
                       0, server,
                       1, OBJ(server)->name,
                       -1);
    
    ui_nap_server_update(server);
}

void ui_nap_server_new(nap_server_t* server)
{
    g_assert(server);
    
    object_notify_d(OBJ(server), (ObjectFunc)ui_nap_server_hide, NULL);
    object_monitor_d(OBJ(server), (ObjectFunc)ui_nap_server_update, NULL);
    
    ui_nap_server_show(server);
}

static void ui_nap_net_update(nap_net_t* net)
{
    GtkTreeIter* node;
    int gigs = 0;
    int files = 0;
    int users = 0;
    const char* status = NULL;
    int diff=0;
    
    if (!net) return;

    node = object_get_data(OBJ(net), "NapNetEntry");

    // check whether node is shown
    if (!node)
    {
        ui_nap_net_show(net);
        return;
    }
    else if (!net->group)
    {
        ui_nap_net_hide(net);
        return;
    }

    // check whether the node has changed its parent
    /*
      if (!ui_iter_is_parent_to(GTK_TREE_MODEL(ui_nap_get_net_tree()), parent, node)) {
      g_assert(FALSE);
      ui_nap_net_hide(net);
      ui_nap_net_show(net);
      return;
      }
    */
    
    status = NULL;
    if (net->active && net->active->status == NAP_SERVER_ONLINE)
    {
        status = _("Online");
    }
    else if (net->active)
    {
        status = _("Connecting");
    }
    else if (net->flags & NAP_NET_AUTO_CONNECT)
    {
        status = _("Autoconnect");
    }
    
    users = nap_net_stats(net, 0);
    files = nap_net_stats(net, 1);
    gigs  = nap_net_stats(net, 2);
    
    if (net->last_success)
        diff = CurrentTime - net->last_success;
    
    gtk_tree_store_set(ui_nap_get_net_tree(), node,
                       1, OBJ(net)->name,  //FIXME
                       2, (net->active && net->active->status == NAP_SERVER_ONLINE),
                       3, status,
                       4, users,
                       5, files,
                       6, gigs,
                       8, (net->flags & NAP_NET_PERSISTENT)?ServerPerm:NULL,
                       9, net->number_fails,
                       10, diff>0?l_printf(_("since %s"), print_duration_unit(&diff)):"",
                       -1);
}

static void ui_nap_net_show(nap_net_t* net)
{
    GtkTreeIter* node;
    GtkTreeIter* parent;
    GList* dlist;

    if (!net || !net->group) return;

    parent = object_get_data(OBJ(net->group), "NapGroupEntry");
    if (!parent) return;

    node = object_get_data(OBJ(net), "NapNetEntry");
    
    if (!node)
    {
        object_set_data(OBJ(net), "NapNetEntry", node = g_malloc(sizeof(*node)));
        gtk_tree_store_append(ui_nap_get_net_tree(), node, parent);
        gtk_tree_store_set(ui_nap_get_net_tree(), node,
                           0, net,
                           1, OBJ(net)->name,
                           -1);
        ui_nap_net_update(net);
    }
    
    for (dlist = net->servers; dlist; dlist = dlist->next)
    {
        nap_server_t* server = dlist->data;
        if (!object_get_data(OBJ(server), "NapServerEntry"))
            ui_nap_server_show(server);
    }
}

void ui_nap_net_new(nap_net_t* net)
{
    g_assert(net);
    
    object_notify_d(OBJ(net), (ObjectFunc)ui_nap_net_hide, NULL);
    object_monitor_d(OBJ(net), (ObjectFunc)ui_nap_net_update, NULL);
    
    ui_nap_net_show(net);
}

static void ui_nap_ng_update(nap_ng_t* ng)
{
    GtkTreeIter* node;
    int gigs = 0;
    int files = 0;
    int users = 0;
    char* str;
    
    g_assert(ng);
    
    node = object_get_data(OBJ(ng), "NapGroupEntry");

    if (!node)
    {
        ui_nap_ng_show(ng);
        return;
    }
  
    if (ng->online > 0)
    {
        users = nap_ng_stats(ng, 0);
        files = nap_ng_stats(ng, 1);
        gigs  = nap_ng_stats(ng, 2);
    }
    
    if (ng->online)
    {
        str =
            l_printf(_("%d/%d connected"), ng->online,
                     (ng->keep==-1)?(int)g_list_length(ng->nets):ng->keep);
    }
    else
    {
        str = NULL;
    }
    gtk_tree_store_set(ui_nap_get_net_tree(), node,
                       1, OBJ(ng)->name,  //FIXME
                       3, str,
                       4, users,
                       5, files,
                       6, gigs,
                       -1);
}

static void ui_nap_ng_show(nap_ng_t* ng)
{
    GtkTreeIter* node;
    GList* dlist;
    
    g_assert(ng);
    
    node = object_get_data(OBJ(ng), "NapGroupEntry");

    if (!node)
    {
        object_set_data(OBJ(ng), "NapGroupEntry", node = g_malloc(sizeof(*node)));
        gtk_tree_store_append(ui_nap_get_net_tree(), node, NULL);
        gtk_tree_store_set(ui_nap_get_net_tree(), node,
                           0, ng,
                           1, OBJ(ng)->name,
                           -1);
        ui_nap_ng_update(ng);
    }
    
    for (dlist = ng->nets; dlist; dlist = dlist->next)
    {
        nap_net_t* net = dlist->data;
        if (!object_get_data(OBJ(net), "NapNetEntry")) ui_nap_net_show(net);
    }
}

void ui_nap_ng_new(nap_ng_t* ng)
{
    g_assert(ng);
    
    object_notify_d(OBJ(ng), (ObjectFunc)ui_nap_ng_hide, NULL);
    object_monitor_d(OBJ(ng), (ObjectFunc)ui_nap_ng_update, NULL);
    
    ui_nap_ng_show(ng);
}

static void on_net_edit_clicked(GObject* button, nap_net_t* net)
{
    GtkEntry* entry;
    GtkWidget* item;
    GtkToggleButton* tbutton;
    GtkWidget* win;
    const char* name;
    const char* user;
    const char* pass;
    nap_ng_t* ng;
    
    entry = g_object_get_data(button, "nick");
    user = ui_entry_get_text(entry);
    if (strchr(user, ' '))
    {
        lopster_notify(LOPSTER_MESSAGE_ERROR, _("Invalid input"),
                       _("The username may not contain any spaces"));
        return;
    }
    entry = g_object_get_data(button, "pass");
    pass = ui_entry_get_text(entry);
    if (strchr(pass, ' '))
    {
        lopster_notify(LOPSTER_MESSAGE_ERROR, _("Invalid input"),
                       _("The password may not contain any spaces"));
        return;
    }
    
    entry = g_object_get_data(button, "name");
    name = ui_entry_get_text(entry);
    if (*name == 0)
    {
        lopster_notify(LOPSTER_MESSAGE_ERROR, _("Invalid input"),
                       _("The network name is empty"));
        return;
    }
    if (!net)
    {
        ng = g_object_get_data(button, "group");
        net = nap_net_create(ng, name, NAP_CREATE_UNIQUE);
        if (!net)
        {
            lopster_notify(LOPSTER_MESSAGE_ERROR, _("Invalid input"),
                           _("This group already contains a network with this name"));
            return;
        }
    }
    else
    {
        object_set_name(OBJ(net), name);
    }

    nap_net_set_nick(net, user);
    nap_net_set_password(net, pass);
    
    entry = g_object_get_data(button, "join");
    make_list_from_string(&net->autojoin, ui_entry_get_text(entry), " ");
    
    tbutton = g_object_get_data(button, "server");
    if (gtk_toggle_button_get_active(tbutton))
        net->flags |= NAP_NET_OWN_SERVER;
    else
        net->flags &= ~NAP_NET_OWN_SERVER;
    
    tbutton = g_object_get_data(button, "global");
    if (gtk_toggle_button_get_active(tbutton))
        net->flags |= NAP_NET_IGN_GLOBAL;
    else
        net->flags &= ~NAP_NET_IGN_GLOBAL;
    
    tbutton = g_object_get_data(button, "winmx");
    if (gtk_toggle_button_get_active(tbutton))
        net->flags |= NAP_NET_NO_WINMX;
    else
        net->flags &= ~NAP_NET_NO_WINMX;
    
    tbutton = g_object_get_data(button, "keep");
    if (gtk_toggle_button_get_active(tbutton))
        net->flags |= NAP_NET_PERSISTENT;
    else
        net->flags &= ~NAP_NET_PERSISTENT;
    
    
    item = g_object_get_data(button, "charset");
    g_free(net->charset);
    net->charset = g_strdup(ui_charset_control_get_charset(item));
    net->charset_warned = FALSE;
    
    object_set_dirty(OBJ(net));
    nap_network_save();
    
    win = g_object_get_data(button, "win");
    gtk_widget_destroy(win);
}

static void
start_con_func(GtkTreeModel* model, GtkTreePath* path, GtkTreeIter* iter)
{
    nap_net_t* net;
    
    if (gtk_tree_path_get_depth(path) != 2) return;
    gtk_tree_model_get (model, iter, 0, &net, -1);
    
    nap_net_start_connect(net);
}

static void
stop_con_func(GtkTreeModel* model, GtkTreePath* path, GtkTreeIter* iter)
{
    nap_net_t* net;
    
    if (gtk_tree_path_get_depth(path) != 2) return;
    gtk_tree_model_get (model, iter, 0, &net, -1);
    nap_net_stop_connect(net);
}

static void
dis_con_func(GtkTreeModel* model, GtkTreePath* path, GtkTreeIter* iter)
{
    nap_net_t* net;
    
    if (gtk_tree_path_get_depth(path) != 2) return;
    gtk_tree_model_get (model, iter, 0, &net, -1);
    
    nap_net_stop_connect(net);
    nap_net_disconnect(net);
}

static void on_show_channels(nap_net_t* net)
{
    if (!net) return;
    nap_net_show_channels(net);
}

/* static void on_global_list(nap_net_t* net) { */
/*   if (!net) return; */
/*   nap_net_get_global_users(net, NULL); */
/* } */

static void
toggle_winmx_func(GtkTreeModel* model, GtkTreePath* path,
                  GtkTreeIter* iter) {
  nap_net_t* net;

  if (gtk_tree_path_get_depth(path) != 2) return;
  gtk_tree_model_get (model, iter, 0, &net, -1);
  net->flags ^= NAP_NET_NO_WINMX;
  object_set_dirty(OBJ(net));
}
static void on_net_toggle_winmx(GtkTreeSelection* select) {
  if (!select) return;
  gtk_tree_selection_selected_foreach
    (select, (GtkTreeSelectionForeachFunc)toggle_winmx_func, NULL);
  nap_network_save();
}

static void
toggle_server_func(GtkTreeModel* model, GtkTreePath* path,
                   GtkTreeIter* iter) {
  nap_net_t* net;

  if (gtk_tree_path_get_depth(path) != 2) return;
  gtk_tree_model_get (model, iter, 0, &net, -1);
  net->flags ^= NAP_NET_OWN_SERVER;
  object_set_dirty(OBJ(net));
}
static void on_net_toggle_server(GtkTreeSelection* select) {

  if (!select) return;
  gtk_tree_selection_selected_foreach
    (select, (GtkTreeSelectionForeachFunc)toggle_server_func, NULL);
  nap_network_save();
}

static void
toggle_global_func(GtkTreeModel* model, GtkTreePath* path, GtkTreeIter* iter) {
  nap_net_t* net;

  if (gtk_tree_path_get_depth(path) != 2) return;
  gtk_tree_model_get (model, iter, 0, &net, -1);
  net->flags ^= NAP_NET_IGN_GLOBAL;
  object_set_dirty(OBJ(net));
}
static void on_net_toggle_global(GtkTreeSelection* select) {
  if (!select) return;
  gtk_tree_selection_selected_foreach
    (select, (GtkTreeSelectionForeachFunc)toggle_global_func, NULL);
  nap_network_save();
}


static void
toggle_persistent_func(GtkTreeModel* model, GtkTreePath* path, GtkTreeIter* iter) {
  nap_net_t* net;

  if (gtk_tree_path_get_depth(path) != 2) return;
  gtk_tree_model_get(model, iter, 0, &net, -1);
  net->flags ^= NAP_NET_PERSISTENT;
  object_set_dirty(OBJ(net));
}

static void on_net_toggle_pers(GtkTreeSelection* select) {
  if (!select) return;
  gtk_tree_selection_selected_foreach
    (select, (GtkTreeSelectionForeachFunc)toggle_persistent_func, NULL);
  nap_network_save();
}



static void net_move_func(GtkTreeRowReference* ref, nap_ng_t* ng) {
  GtkTreeIter iter;
  GtkTreePath* path;
  GtkTreeModel* model;
  nap_net_t* net;

  model = GTK_TREE_MODEL(ui_nap_get_net_tree());

  path = gtk_tree_row_reference_get_path(ref);
  if (gtk_tree_path_get_depth(path) != 2) {
    gtk_tree_path_free(path);
    return;
  }
  if (!gtk_tree_model_get_iter(model, &iter, path)) return;
  gtk_tree_path_free(path);
  gtk_tree_model_get (model, &iter, 0, &net, -1);

  if (!net || net->group == ng) return;

  if (!ng) {
    object_kill(OBJ(net));
  } else {
    nap_net_move(net, ng);
  }
}

static void on_net_move(GtkMenuItem* item, GtkTreeSelection* select) {
  GList* selection;
  GtkTreeModel* model;
  GList* dlist;
  GtkTreePath* path;
  GList* refs = NULL;
  GtkTreeRowReference* ref;
  nap_ng_t* ng;

  model = GTK_TREE_MODEL(ui_nap_get_net_tree());
  selection = gtk_tree_selection_get_selected_rows(select, &model);
  // convert paths to references
  for (dlist = selection; dlist; dlist = dlist->next) {
    path = dlist->data;
    ref = gtk_tree_row_reference_new(model, path);
    refs = g_list_append(refs, ref);
  }
  // free paths
  g_list_foreach(selection, (GFunc)gtk_tree_path_free, NULL);
  g_list_free(selection);

  // process the references
  ng = g_object_get_data(G_OBJECT(item), "dest");
  g_list_foreach(refs, (GFunc)net_move_func, ng);

  // free references
  g_list_foreach(refs, (GFunc)gtk_tree_row_reference_free, model);
  g_list_free(refs);
}

static void on_server_refresh(GtkBin* entry)
{
    nap_service_get_list(ui_entry_get_text(GTK_ENTRY(entry->child)));
}

static void on_group_edit_clicked(GObject* button, nap_ng_t* ng)
{
    GtkEntry* entry;
    GtkToggleButton* tbutton;
    GtkSpinButton* sbutton;
    GtkWidget* win;
    const char* name;
    
    entry = g_object_get_data(button, "name");
    name = ui_entry_get_text(entry);
    if (!name || *name == 0)
    {
        lopster_notify(LOPSTER_MESSAGE_INFO, _("Invalid input"),
                       _("The group name is empty"));
        return;
    }
  
    if (!ng)
    {
        ng = nap_ng_create(name, 0, NAP_CREATE_UNIQUE);
        if (!ng)
        {
            lopster_notify(LOPSTER_MESSAGE_INFO, _("Invalid input"),
                           _("A group with this name already exists"));
            return;
        }
    }
    else
    {
        object_set_name(OBJ(ng), name);
    }

    sbutton = g_object_get_data(button, "keep");
    ng->keep = gtk_spin_button_get_value(sbutton);
    tbutton = g_object_get_data(button, "never");
    if (gtk_toggle_button_get_active(tbutton))
        ng->flags |= NAP_NG_AUTO_CONNECT;
    else
        ng->flags &= ~NAP_NG_AUTO_CONNECT;
    
    object_set_dirty(OBJ(ng));
    nap_network_save();
    
    win = g_object_get_data(button, "win");
    gtk_widget_destroy(win);
}

static void on_create_edit_group(nap_ng_t* group) {
  GtkWidget *win;
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *entry;
  GtkWidget *spin;
  GtkWidget *button;
  GtkWidget* item;
  GtkWidget* checkbutton;
  GtkWidget* hbox2;

  win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW (win), group?_("Edit Group"):_("Create Group"));
/*   gtk_window_set_default_size(GTK_WINDOW(win), 300, -1); */
  gtk_window_set_type_hint(GTK_WINDOW(win), GDK_WINDOW_TYPE_HINT_DIALOG);
  gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_CENTER_ON_PARENT);
  if (ui_main_get_win())
  {
      gtk_window_set_transient_for(GTK_WINDOW(win), ui_main_get_win());
      gtk_window_set_destroy_with_parent(GTK_WINDOW(win), TRUE);
  }
  
  button = gtk_button_new_from_stock(GTK_STOCK_OK);

  vbox = gtk_vbox_new(FALSE, 10);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
  gtk_container_add(GTK_CONTAINER(win), vbox);

  //
  item = ui_create_section(_("Group settings"), NULL, &hbox, NULL);
  gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);

  hbox2 = gtk_hbox_new(FALSE, 10);
  gtk_box_pack_start(GTK_BOX(hbox), hbox2, TRUE, TRUE, 0);
  
  entry = ui_entry_new_default(group?OBJ(group)->name:NULL, _("Enter name here"));
  g_object_set_data(G_OBJECT(button), "name", entry);
  item = ui_create_headed_item(_("group name"), entry);
  gtk_box_pack_start(GTK_BOX(hbox2), item, TRUE, TRUE, 0);

  spin = gtk_spin_button_new_with_range(-1, group?g_list_length(group->nets):100, 1);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), group?group->keep:5);
  g_object_set_data(G_OBJECT(button), "keep", spin);

  item = ui_create_headed_item(_("connections to keep"), spin);
  gtk_box_pack_start(GTK_BOX(hbox2), item, FALSE, FALSE, 0);

  //
  item = ui_create_section(_("Group flags"), NULL, &hbox, NULL);
  gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);

  checkbutton = gtk_check_button_new_with_label(_("Include in autoconnect"));
  if (group && (group->flags & NAP_NG_AUTO_CONNECT))
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), TRUE);
  g_object_set_data(G_OBJECT(button), "never", checkbutton);
  gtk_box_pack_start(GTK_BOX(hbox), checkbutton, FALSE, FALSE, 0);


  ////////////////
  hbox = ui_create_bbox(10, GTK_BUTTONBOX_END);
  gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

  {
    GtkWidget* button2 = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
    gtk_container_add(GTK_CONTAINER(hbox), button2);
    g_signal_connect_swapped(G_OBJECT(button2), "clicked",
                             G_CALLBACK(gtk_widget_destroy), win);
  }

  gtk_container_add(GTK_CONTAINER(hbox), button);
  g_signal_connect(G_OBJECT(button), "clicked",
                   G_CALLBACK(on_group_edit_clicked), group);
  g_object_set_data(G_OBJECT(button), "win", win);


  gtk_widget_show_all(win);
}

static void ui_menu_add_top(GtkWidget* popup) {
  GtkWidget* item;

  item = gtk_menu_item_new_with_label(_("Create Group"));
  gtk_widget_show(item);
  gtk_container_add(GTK_CONTAINER(popup), item);
  g_signal_connect_swapped(G_OBJECT(item), "activate",
                           G_CALLBACK(on_create_edit_group), NULL);

  item = gtk_menu_item_new();
  gtk_widget_show(item);
  gtk_container_add(GTK_CONTAINER(popup), item);
  gtk_widget_set_sensitive(item, FALSE);
}

static void on_server_disconnect(nap_server_t* server) {
  nap_net_disconnect(server->net);
}

static void on_server_connect(nap_server_t* server) {
  nap_server_connect(server);
}

static void
on_server_edit_clicked(GObject* button, nap_server_t* server) {
  GtkEntry* entry;
  GtkToggleButton* tbutton;
  GtkSpinButton* sbutton;
  GtkWidget* win;
  nap_net_t* net;
  const char* name;

  entry = g_object_get_data(button, "address");
  name = ui_entry_get_text(entry);

  if (!name || *name == 0) {
    lopster_notify(LOPSTER_MESSAGE_ERROR, _("Invalid input"),
                   _("The server address is empty"));
    return;
  }

  if (!server) {
    net = g_object_get_data(button, "net");
    server = nap_server_create(net, name, NAP_CREATE_ALWAYS);
  } else {
    object_set_name(OBJ(server), name);
    server->ip = 0;
  }

  sbutton = g_object_get_data(button, "port");
  server->port = gtk_spin_button_get_value(sbutton);

  tbutton = g_object_get_data(button, "meta");
  if (gtk_toggle_button_get_active(tbutton))
    server->flags |= NAP_SERVER_META;
  else
    server->flags &= ~NAP_SERVER_META;

  tbutton = g_object_get_data(button, "inactive");
  if (gtk_toggle_button_get_active(tbutton))
    server->flags |= NAP_SERVER_IGNORE;
  else
    server->flags &= ~NAP_SERVER_IGNORE;

  object_set_dirty(OBJ(server));
  nap_network_save();

  win = g_object_get_data(button, "win");
  gtk_widget_destroy(win);
}

static void on_create_edit_server(nap_server_t* server, nap_net_t* net) {
  GtkWidget* win;
  GtkWidget* vbox;
  GtkWidget* hbox;
  GtkWidget* entry;
  GtkWidget* spin;
  GtkWidget* item;
  GtkWidget* hbox2;
  GtkWidget* vbox2;
  GtkWidget* button;

  win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  if (server) {
    gtk_window_set_title (GTK_WINDOW (win), _("Edit Server"));
  } else {
    gtk_window_set_title (GTK_WINDOW (win), _("Create Server"));
  }
  gtk_window_set_type_hint(GTK_WINDOW(win), GDK_WINDOW_TYPE_HINT_DIALOG);
  gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_CENTER_ON_PARENT);
  if (ui_main_get_win())
  {
      gtk_window_set_transient_for(GTK_WINDOW(win), ui_main_get_win());
      gtk_window_set_destroy_with_parent(GTK_WINDOW(win), TRUE);
  }

  button = gtk_button_new_from_stock(GTK_STOCK_OK);
  g_object_set_data(G_OBJECT(button), "net", net);
  
  vbox = gtk_vbox_new(FALSE, 10);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
  gtk_container_add(GTK_CONTAINER(win), vbox);

  //
  item = ui_create_section(_("Server settings"), NULL, &hbox, NULL);
  gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);

  hbox2 = gtk_hbox_new(FALSE, 10);
  gtk_box_pack_start(GTK_BOX(hbox), hbox2, TRUE, TRUE, 0);

  entry = ui_entry_new_default(server?OBJ(server)->name:NULL, _("Enter address here"));
  g_object_set_data(G_OBJECT(button), "address", entry);
  item = ui_create_headed_item(_("address"), entry);
  gtk_box_pack_start(GTK_BOX(hbox2), item, TRUE, TRUE, 0);

  spin = gtk_spin_button_new_with_range(0, 65535, 1);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), server?server->port:8888);
  gtk_spin_button_set_increments(GTK_SPIN_BUTTON(spin), 1, 100);
  g_object_set_data(G_OBJECT(button), "port", spin);
  item = ui_create_headed_item(_("port"), spin);
  gtk_box_pack_start(GTK_BOX(hbox2), item, FALSE, FALSE, 0);


  //
  item = ui_create_section(_("Server flags"), NULL, &hbox, NULL);
  gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);

  vbox2 = gtk_vbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, TRUE, 0);

  item = gtk_check_button_new_with_label(_("Server redirects (meta server)"));
  if (server && (server->flags&NAP_SERVER_META))
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(item), TRUE);
  g_object_set_data(G_OBJECT(button), "meta", item);
  gtk_box_pack_start(GTK_BOX(vbox2), item, FALSE, FALSE, 0);

  item = gtk_check_button_new_with_label(_("Deactivated"));
  if (server && (server->flags & NAP_SERVER_IGNORE))
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(item), TRUE);
  g_object_set_data(G_OBJECT(button), "inactive", item);
  gtk_box_pack_start(GTK_BOX(vbox2), item, FALSE, FALSE, 0);

  //
  hbox = ui_create_bbox(10, GTK_BUTTONBOX_END);
  gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

  {
    GtkWidget* button2 = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
    gtk_container_add(GTK_CONTAINER(hbox), button2);
    g_signal_connect_swapped(G_OBJECT(button2), "clicked",
                             G_CALLBACK(gtk_widget_destroy), win);
  }

  gtk_container_add(GTK_CONTAINER(hbox), button);
  g_signal_connect(G_OBJECT(button), "clicked",
                   G_CALLBACK(on_server_edit_clicked), server);
  g_object_set_data(G_OBJECT(button), "win", win);

      
  gtk_widget_show_all(win);
}

static void on_edit_server(nap_server_t* server) {
  on_create_edit_server(server, NULL);
}

static void on_server_toggle_ignore(nap_server_t* server) {
  if (server) {
    server->flags ^= NAP_SERVER_IGNORE;
    object_set_dirty(OBJ(server));
    nap_network_save();
  }
}

static GtkWidget*
ui_create_server_popup(GtkWidget* popup, nap_server_t* server) {
  GtkWidget *item;
  int server_status;
  nap_net_t* net;
  GtkWidget* popup2;

  if (!popup) popup = gtk_menu_new();

  net = server->net;
  server_status = server->status;
  
  item = gtk_menu_item_new_with_label(_("Server action"));
  gtk_widget_show(item);
  gtk_container_add(GTK_CONTAINER(popup), item);
  {
    popup2 = gtk_menu_new();
    gtk_widget_show(popup2);
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), popup2);
    
    if (server_status == NAP_SERVER_CONNECTING ||
        server_status == NAP_SERVER_ONLINE) {
      item = gtk_menu_item_new_with_label(_("Disconnect"));
      gtk_widget_show(item);
      gtk_container_add(GTK_CONTAINER(popup2), item);
      g_signal_connect_swapped(G_OBJECT(item), "activate",
                               G_CALLBACK(on_server_disconnect), 
                               server);
    } else {
      item = gtk_menu_item_new_with_label(_("Connect"));
      gtk_widget_show(item);
      gtk_container_add(GTK_CONTAINER(popup2), item);
      g_signal_connect_swapped(G_OBJECT(item), "activate",
                               G_CALLBACK(on_server_connect), 
                               server);
    }

    item = gtk_menu_item_new();
    gtk_widget_show(item);
    gtk_container_add(GTK_CONTAINER(popup2), item);
    gtk_widget_set_sensitive(item, FALSE);
    
    item = gtk_menu_item_new_with_label(_("Edit Server"));
    gtk_widget_show(item);
    gtk_container_add(GTK_CONTAINER(popup2), item);
    g_signal_connect_swapped(G_OBJECT(item), "activate",
                             G_CALLBACK(on_edit_server), server);

    item = gtk_menu_item_new_with_label(_("Remove Server"));
    gtk_widget_show(item);
    gtk_container_add(GTK_CONTAINER(popup2), item);

    g_signal_connect_swapped(G_OBJECT(item), "activate",
                             G_CALLBACK(object_kill), server);
  }  

  item = gtk_menu_item_new_with_label(_("Quick configure"));
  gtk_widget_show(item);
  gtk_container_add(GTK_CONTAINER(popup), item);
  {
    popup2 = gtk_menu_new();
    gtk_widget_show(popup2);
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), popup2);

    item = gtk_check_menu_item_new_with_label(_("Deactivated"));
    if (server->flags & NAP_SERVER_IGNORE)
      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), TRUE);
    gtk_widget_show(item);
    gtk_container_add(GTK_CONTAINER(popup2), item);
    g_signal_connect_swapped(G_OBJECT(item), "activate",
                             G_CALLBACK(on_server_toggle_ignore), 
                             server);
  }
  
  return popup;
}

static GtkWidget* ui_create_server1_popup(nap_server_t* server) {
  GtkWidget* popup = gtk_menu_new();

  ui_menu_add_top(popup);

  if (server) ui_create_server_popup(popup, server);

  return popup;
}

static void on_create_edit_net(nap_net_t* net, nap_ng_t* ng) {
  GtkWidget* win;
  GtkWidget* vbox;
  GtkWidget* hbox;
  GtkWidget* entry;
  GtkWidget* button;
  GtkWidget* item;
  GtkWidget* vbox2;

  win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  if (net) {
    gtk_window_set_title (GTK_WINDOW (win), _("Edit Network"));
  } else {
    gtk_window_set_title (GTK_WINDOW (win), _("Create Network"));
  }
  gtk_window_set_type_hint(GTK_WINDOW(win), GDK_WINDOW_TYPE_HINT_DIALOG);
  gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_CENTER_ON_PARENT);
  if (ui_main_get_win())
  {
      gtk_window_set_transient_for(GTK_WINDOW(win), ui_main_get_win());
      gtk_window_set_destroy_with_parent(GTK_WINDOW(win), TRUE);
  }

  button = gtk_button_new_from_stock(GTK_STOCK_OK);
  g_object_set_data(G_OBJECT(button), "group", ng);
  
  vbox = gtk_vbox_new(FALSE, 10);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
  gtk_container_add(GTK_CONTAINER(win), vbox);


  //
  item = ui_create_section(_("Network settings"), NULL, &hbox, NULL);
  gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);
  
  vbox2 = gtk_hbox_new(FALSE, 10);
  gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, TRUE, 0);
  
  entry = ui_entry_new_default(net?OBJ(net)->name:NULL, _("Enter name here"));
  g_object_set_data(G_OBJECT(button), "name", entry);
  item = ui_create_headed_item(_("network name"), entry);
  gtk_box_pack_start(GTK_BOX(vbox2), item, FALSE, FALSE, 0);

  entry = ui_entry_new(NULL);
  if (net)
  {
      char* str = make_string_from_list(net->autojoin, " ");
      ui_entry_set_text(GTK_ENTRY(entry), str);
      g_free(str);
  }
  g_object_set_data(G_OBJECT(button), "join", entry);
  item = ui_create_headed_item(_("autojoin channels"), entry);
  gtk_box_pack_start(GTK_BOX(vbox2), item, FALSE, FALSE, 0);
  

  //
  item = ui_create_section(_("Network flags"), NULL, &hbox, NULL);
  gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);

  vbox2 = gtk_vbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, TRUE, 0);
  
  item = gtk_check_button_new_with_label(_("Don't remove servers automactically"));
  if (!net || (net->flags & NAP_NET_PERSISTENT))
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(item), TRUE);
  g_object_set_data(G_OBJECT(button), "keep", item);
  gtk_box_pack_start(GTK_BOX(vbox2), item, FALSE, FALSE, 0);

  item = gtk_check_button_new_with_label(_("Own server page"));
  if (net && (net->flags & NAP_NET_OWN_SERVER))
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(item), TRUE);
  g_object_set_data(G_OBJECT(button), "server", item);
  gtk_box_pack_start(GTK_BOX(vbox2), item, FALSE, FALSE, 0);

  item = gtk_check_button_new_with_label(_("Ignore global messages"));
  if (net && (net->flags & NAP_NET_IGN_GLOBAL))
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(item), TRUE);
  g_object_set_data(G_OBJECT(button), "global", item);
  gtk_box_pack_start(GTK_BOX(vbox2), item, FALSE, FALSE, 0);

  item = gtk_check_button_new_with_label(_("Don't emulate WinMX 2.6"));
  if (net && (net->flags & NAP_NET_NO_WINMX))
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(item), TRUE);
  g_object_set_data(G_OBJECT(button), "winmx", item);
  gtk_box_pack_start(GTK_BOX(vbox2), item, FALSE, FALSE, 0);


  //
  item = ui_create_section(_("Override standard settings"), NULL, &hbox, NULL);
/*   item = ui_create_expander_section(_("Override standard settings"), NULL, &hbox); */
  gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);

  vbox2 = gtk_vbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, TRUE, 0);

  hbox = gtk_hbox_new(FALSE, 10);
  gtk_box_pack_start(GTK_BOX(vbox2), hbox, TRUE, TRUE, 0);

  entry = ui_entry_new_default(net && net->nick?net->nick:NULL, _("Using default username"));
  g_object_set_data(G_OBJECT(button), "nick", entry);
  item = ui_create_headed_item(_("login"), entry);
  gtk_box_pack_start(GTK_BOX(hbox), item, TRUE, TRUE, 0);

  entry = ui_entry_new_default_pass(net && net->pass?net->pass:NULL, _("Using default password"));
  g_object_set_data(G_OBJECT(button), "pass", entry);
  item = ui_create_headed_item(_("password"), entry);
  gtk_box_pack_start(GTK_BOX(hbox), item, TRUE, TRUE, 0);
  
  entry = ui_create_charset_control(net?net->charset:NULL, NULL, NULL);
  g_object_set_data(G_OBJECT(button), "charset", entry);
  item = ui_create_headed_item(_("character encoding"), entry);
  gtk_box_pack_start(GTK_BOX(vbox2), item, FALSE, FALSE, 0);



  ////////////////
  hbox = ui_create_bbox(10, GTK_BUTTONBOX_END);
  gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

  {
    GtkWidget* button2 = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
    gtk_container_add(GTK_CONTAINER(hbox), button2);
    g_signal_connect_swapped(G_OBJECT(button2), "clicked",
                             G_CALLBACK(gtk_widget_destroy), win);
  }

  gtk_container_add(GTK_CONTAINER(hbox), button);
  g_signal_connect(G_OBJECT(button), "clicked",
                   G_CALLBACK(on_net_edit_clicked), net);
  g_object_set_data(G_OBJECT(button), "win", win);

  gtk_widget_show_all(win);
}

static void on_create_net(nap_ng_t* ng) {
  on_create_edit_net(NULL, ng);
}
static void on_ng_toggle_auto(nap_ng_t* ng) {
  ng->flags ^= NAP_NG_AUTO_CONNECT;
  object_set_dirty(OBJ(ng));
  nap_network_save();
}

static void on_keep(GtkMenuItem* item, nap_ng_t* ng) {
  int data = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(item), "hm"));
  ng->keep = data;
  object_set_dirty(OBJ(ng));
  nap_network_save();
}

static GtkWidget*
create_netgroup_popup(GtkWidget* popup, nap_ng_t* ng) {
#define SUB_LEN  20
  GtkWidget *item;
  GtkWidget *popup2;
  GtkWidget *popup3;
  int i1;
  GtkWidget* items[SUB_LEN+1];
  int no_nets, inc;
  char* str;

  if (!popup) popup = gtk_menu_new();

  no_nets = g_list_length(ng->nets);
  inc = (no_nets+SUB_LEN-1)/SUB_LEN;
  
  item = gtk_menu_item_new_with_label(_("Group action"));
  gtk_widget_show(item);
  gtk_container_add(GTK_CONTAINER(popup), item);
  
  {
    popup2 = gtk_menu_new();
    gtk_widget_show(popup2);
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), popup2);

    if (ng->nets) {
      item = gtk_menu_item_new_with_label(_("Start connect"));
      gtk_widget_show(item);
      gtk_container_add(GTK_CONTAINER(popup2), item);
      g_signal_connect_swapped(G_OBJECT(item), "activate",
                               G_CALLBACK(nap_ng_start_connect), ng);
      item = gtk_menu_item_new_with_label(_("Stop connect"));
      gtk_widget_show(item);
      gtk_container_add(GTK_CONTAINER(popup2), item);
      g_signal_connect_swapped(G_OBJECT(item), "activate",
                               G_CALLBACK(nap_ng_stop_connect), ng);
    } else {
      item = gtk_menu_item_new_with_label(_("Remove Group"));
      gtk_widget_show(item);
      gtk_container_add(GTK_CONTAINER(popup2), item);
      g_signal_connect_swapped(G_OBJECT(item), "activate",
                               G_CALLBACK(object_kill), ng);
    }

    item = gtk_menu_item_new();
    gtk_widget_show(item);
    gtk_container_add(GTK_CONTAINER(popup2), item);
    gtk_widget_set_sensitive(item, FALSE);
    
    item = gtk_menu_item_new_with_label(_("Edit Group"));
    gtk_widget_show(item);
    gtk_container_add(GTK_CONTAINER(popup2), item);
    g_signal_connect_swapped(G_OBJECT(item), "activate",
                             G_CALLBACK(on_create_edit_group), ng);
    
    item = gtk_menu_item_new_with_label(_("Add Network"));
    gtk_widget_show(item);
    gtk_container_add(GTK_CONTAINER(popup2), item);
    g_signal_connect_swapped(G_OBJECT(item), "activate",
                             G_CALLBACK(on_create_net), ng);
  }

  item = gtk_menu_item_new_with_label(_("Quick configure"));
  gtk_widget_show(item);
  gtk_container_add(GTK_CONTAINER(popup), item);

  {
    popup2 = gtk_menu_new();
    gtk_widget_show(popup2);
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), popup2);

    item = gtk_menu_item_new_with_label(_("Stay connected to"));
    gtk_widget_show(item);
    gtk_container_add(GTK_CONTAINER(popup2), item);
    {
      popup3 = gtk_menu_new();
      gtk_widget_show(popup3);
      gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), popup3);
      
      if (ng->keep == -1) str = l_strdup(_("All networks"));
      else str = l_printf(ngettext("%d network", "%d networks", ng->keep), ng->keep);

      item = gtk_menu_item_new_with_label(str);
      gtk_widget_show(item);
      gtk_widget_set_sensitive(item, FALSE);
      gtk_container_add(GTK_CONTAINER(popup3), item);
      
      item = gtk_menu_item_new();
      gtk_widget_show(item);
      gtk_container_add(GTK_CONTAINER(popup3), item);
      
      items[0] = gtk_menu_item_new_with_label(_("All networks"));
      gtk_widget_show (items[0]);
      gtk_container_add (GTK_CONTAINER (popup3), items[0]);
      g_object_set_data(G_OBJECT(items[0]), "hm", (void*)-1);
      
      for (i1 = 0; i1*inc < no_nets; i1++) {
        str = l_printf(ngettext("%d network", "%d networks", i1*inc), i1*inc);
        items[i1+1] = gtk_menu_item_new_with_label (str);
        gtk_widget_show(items[i1+1]);
        gtk_container_add(GTK_CONTAINER(popup3), items[i1+1]);
        g_object_set_data(G_OBJECT(items[i1+1]), "hm", GINT_TO_POINTER((i1*inc)));
      }    
    }
    g_signal_connect(G_OBJECT(items[0]), "activate",
                     G_CALLBACK(on_keep), ng);
    for (i1 = 0; i1*inc < no_nets; i1++) {
      g_signal_connect(G_OBJECT(items[i1+1]), "activate",
                       G_CALLBACK(on_keep), ng);
    }    

    item = gtk_menu_item_new();
    gtk_widget_show(item);
    gtk_container_add(GTK_CONTAINER(popup2), item);
    gtk_widget_set_sensitive(item, FALSE);

    item = gtk_check_menu_item_new_with_label(_("Include in Autoconnect"));
    if (ng->flags & NAP_NG_AUTO_CONNECT)
      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), TRUE);
    gtk_widget_show(item);
    gtk_container_add(GTK_CONTAINER(popup2), item);
    g_signal_connect_swapped(G_OBJECT(item), "activate",
                             G_CALLBACK(on_ng_toggle_auto), ng);
  }

  return popup;
}

static GtkWidget* ui_create_netgroup1_popup(nap_ng_t* ng) {
  GtkWidget* popup = gtk_menu_new();
  
  ui_menu_add_top(popup);

  if (ng) create_netgroup_popup(popup, ng);

  return popup;
}

static void on_net_stop_connect(GtkTreeSelection* select) {
  if (!select) return;
  gtk_tree_selection_selected_foreach
    (select, (GtkTreeSelectionForeachFunc)stop_con_func, NULL);
}

static void on_net_start_connect(GtkTreeSelection* select) {
  if (!select) return;
  gtk_tree_selection_selected_foreach
    (select, (GtkTreeSelectionForeachFunc)start_con_func, NULL);
}
static void on_net_disconnect(GtkTreeSelection* select) {
  if (!select) return;
  gtk_tree_selection_selected_foreach
    (select, (GtkTreeSelectionForeachFunc)dis_con_func, NULL);
}
static void on_edit_net(nap_net_t* net) {
  on_create_edit_net(net, NULL);
}
static void on_create_server(nap_net_t* net) {
  on_create_edit_server(NULL, net);
}


static GtkWidget* ui_create_network_popup(GtkWidget* popup, nap_net_t* net) {
  GtkWidget* item;
  GtkWidget* popup2;
  GList* dlist;
  nap_ng_t* ng;
  GtkTreeSelection* select;

  if (!popup) popup = gtk_menu_new();

  item = gtk_menu_item_new_with_label(_("Network action"));
  gtk_widget_show(item);
  gtk_container_add(GTK_CONTAINER(popup), item);

  select = gtk_tree_view_get_selection(ServerView);
  
  {
    popup2 = gtk_menu_new();
    gtk_widget_show(popup2);
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), popup2);

    if (net->servers) {
      if (net->flags & NAP_NET_AUTO_CONNECT) {
        item = gtk_menu_item_new_with_label(_("Stop connect"));
        g_signal_connect_swapped(G_OBJECT(item), "activate",
                                 G_CALLBACK(on_net_stop_connect), 
                                 select);
      } else {
        item = gtk_menu_item_new_with_label(_("Start connect"));
        g_signal_connect_swapped(G_OBJECT(item), "activate",
                                 G_CALLBACK(on_net_start_connect), 
                                 select);
      }
      gtk_widget_show(item);
      gtk_container_add(GTK_CONTAINER(popup2), item);

      if (net->active) {
        item = gtk_menu_item_new_with_label(_("Disconnect"));
        gtk_widget_show(item);
        gtk_container_add(GTK_CONTAINER(popup2), item);
        g_signal_connect_swapped(G_OBJECT(item), "activate",
                                 G_CALLBACK(on_net_disconnect), 
                                 select);
      }
      item = gtk_menu_item_new();
      gtk_widget_show(item);
      gtk_container_add(GTK_CONTAINER(popup2), item);
      gtk_widget_set_sensitive(item, FALSE);
    }

    item = gtk_menu_item_new_with_label(_("Edit Network"));
    gtk_widget_show(item);
    gtk_container_add(GTK_CONTAINER(popup2), item);
    g_signal_connect_swapped(G_OBJECT(item), "activate",
                             G_CALLBACK(on_edit_net), net);
  
    item = gtk_menu_item_new_with_label(_("Add Server"));
    gtk_widget_show(item);
    gtk_container_add(GTK_CONTAINER(popup2), item);
    g_signal_connect_swapped(G_OBJECT(item), "activate",
                             G_CALLBACK(on_create_server), net);


    if (net->active && net->active->status == NAP_SERVER_ONLINE) {
      item = gtk_menu_item_new();
      gtk_widget_show(item);
      gtk_container_add(GTK_CONTAINER(popup2), item);
      gtk_widget_set_sensitive(item, FALSE);

      item = gtk_menu_item_new_with_label(_("Show Channel List"));
      gtk_widget_show(item);
      gtk_container_add(GTK_CONTAINER(popup2), item);
      g_signal_connect_swapped(G_OBJECT(item), "activate",
                               G_CALLBACK(on_show_channels), net);

/*       item = gtk_menu_item_new_with_label("Get Global User List"); */
/*       gtk_widget_show(item); */
/*       gtk_container_add(GTK_CONTAINER(popup2), item); */
/*       if (net->level >= NAP_L_MOD) { */
/*         g_signal_connect(G_OBJECT(item), "activate", */
/*                          G_CALLBACK(on_global_list), net); */
/*       } else { */
/*         gtk_widget_set_sensitive(item, FALSE); */
/*       } */
      
/*       if (net->level < NAP_L_MOD) */
/*         gtk_widget_set_sensitive(item, FALSE); */
      
/*       item = gtk_menu_item_new_with_label("Get Banned Users (NI)"); */
/*       gtk_widget_show(item); */
/*       gtk_container_add(GTK_CONTAINER(popup2), item); */
      /*
        gtk_signal_connect(GTK_OBJECT(item), "activate",
        GTK_SIGNAL_FUNC(on_banned_users), net);
      */      
/*       item = gtk_menu_item_new_with_label("Get Client Statistic (NI)"); */
/*       gtk_widget_show(item); */
/*       gtk_container_add(GTK_CONTAINER(popup2), item); */
      /*
        gtk_signal_connect(GTK_OBJECT(item), "activate",
        GTK_SIGNAL_FUNC(on_client_list), net);
      */
/*       if (net->level < NAP_L_MOD) */
/*         gtk_widget_set_sensitive(item, FALSE); */
      
/*       item = gtk_menu_item_new_with_label("Refresh linked servers (NI)"); */
/*       gtk_widget_show(item); */
/*       gtk_container_add(GTK_CONTAINER(popup2), item); */
      /*
        gtk_signal_connect(GTK_OBJECT(item), "activate",
        GTK_SIGNAL_FUNC(on_server_links), net);
      */      
/*       item = gtk_menu_item_new_with_label("Show server stats (NI)"); */
/*       gtk_widget_show(item); */
/*       gtk_container_add(GTK_CONTAINER(popup2), item); */
      /*
        gtk_signal_connect(GTK_OBJECT(item), "activate",
        GTK_SIGNAL_FUNC(on_server_stats), net);
      */
    }
  }

  item = gtk_menu_item_new();
  gtk_widget_show(item);
  gtk_container_add(GTK_CONTAINER(popup), item);
  gtk_widget_set_sensitive(item, FALSE);

  item = gtk_menu_item_new_with_label(_("Quick configure"));
  gtk_widget_show(item);
  gtk_container_add(GTK_CONTAINER(popup), item);

  {
    popup2 = gtk_menu_new();
    gtk_widget_show(popup2);
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), popup2);

    item = gtk_check_menu_item_new_with_label(_("Has persistent servers"));
    if (net->flags & NAP_NET_PERSISTENT)
      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), TRUE);
    gtk_widget_show(item);
    gtk_container_add(GTK_CONTAINER(popup2), item);
    g_signal_connect_swapped(G_OBJECT(item), "activate",
                             G_CALLBACK(on_net_toggle_pers), select);
  
    item = gtk_check_menu_item_new_with_label(_("Don't emulate WinMX v2.6"));
    if (net->flags & NAP_NET_NO_WINMX)
      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), TRUE);
    gtk_widget_show(item);
    gtk_container_add(GTK_CONTAINER(popup2), item);
    g_signal_connect_swapped(G_OBJECT(item), "activate",
                             G_CALLBACK(on_net_toggle_winmx), select);

    item = gtk_check_menu_item_new_with_label(_("Own server page"));
    if (net->flags & NAP_NET_OWN_SERVER)
      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), TRUE);
    gtk_widget_show(item);
    gtk_container_add(GTK_CONTAINER(popup2), item);
    g_signal_connect_swapped(G_OBJECT(item), "activate",
                             G_CALLBACK(on_net_toggle_server), select);
    
    item = gtk_check_menu_item_new_with_label(_("Ignore global messages"));
    if (net->flags & NAP_NET_IGN_GLOBAL)
      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), TRUE);
    gtk_widget_show(item);
    gtk_container_add(GTK_CONTAINER(popup2), item);
    g_signal_connect_swapped(G_OBJECT(item), "activate",
                             G_CALLBACK(on_net_toggle_global), select);
  }

  item = gtk_menu_item_new_with_label(_("Move Network"));
  gtk_widget_show(item);
  gtk_container_add(GTK_CONTAINER(popup), item);
  {
    popup2 = gtk_menu_new();
    gtk_widget_show(popup2);
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), popup2);
    
    item = gtk_menu_item_new_with_label(_("<Delete Network>"));
    g_object_set_data(G_OBJECT(item), "dest", NULL);
    gtk_container_add(GTK_CONTAINER(popup2), item);
    gtk_widget_show(item);
    g_signal_connect(G_OBJECT(item), "activate",
                     G_CALLBACK(on_net_move), select);

    for (dlist = nap_get_groups(); dlist; dlist = dlist->next) {
      ng = dlist->data;
      
      item = gtk_menu_item_new_with_label(OBJ(ng)->name);
      g_object_set_data(G_OBJECT(item), "dest", ng);
      gtk_container_add(GTK_CONTAINER(popup2), item);
      gtk_widget_show(item);
      g_signal_connect(G_OBJECT(item), "activate",
                       G_CALLBACK(on_net_move), select);
      if (net->group == ng) gtk_widget_set_sensitive(item, FALSE);
    }
  }
  
  if (!net->active || net->active->status != NAP_SERVER_ONLINE)
    return popup;
  
  {
    nap_server_t* server = net->active;
    
    item = gtk_menu_item_new_with_label(_("Active Server"));
    gtk_widget_show(item);
    gtk_container_add(GTK_CONTAINER(popup), item);
    
    popup2 = ui_create_server_popup(NULL, server);
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), popup2);
  }

  return popup;
}

static GtkWidget* ui_create_network1_popup(nap_net_t* net) {
  GtkWidget* popup = gtk_menu_new();
  
  ui_menu_add_top(popup);

  if (net) ui_create_network_popup(popup, net);

  return popup;
}


static gboolean
on_nap_server_tree_button(GtkTreeView *tv, GdkEventButton *event) {
  GtkWidget *pop;
  GtkTreePath* path;
  GtkTreeIter iter;
  //  GtkTreeSelection *select;
  nap_ng_t* ng;
  nap_net_t* net;
  nap_server_t* server;
  int depth;
  GtkTreeModel* model;
  int selected;
  GtkTreeSelection* select;

  //  deactivate_auto_afk();

  if (event->window != gtk_tree_view_get_bin_window(tv))
    return FALSE;

  if (event->type != GDK_BUTTON_PRESS) return FALSE;

  model = gtk_tree_view_get_model(tv);

  pop = NULL;
  if (event->button == 3) {
    if (!gtk_tree_view_get_path_at_pos(tv, event->x, event->y,
                                       &path, NULL, NULL, NULL)) {
      pop = ui_create_server1_popup(NULL);
    } else {
      select = gtk_tree_view_get_selection (tv);
      gtk_tree_model_get_iter(model, &iter, path);
      selected = gtk_tree_selection_iter_is_selected(select, &iter);
      depth = gtk_tree_path_get_depth(path);
      gtk_tree_path_free(path);
      
      if (!selected) {
        gtk_tree_selection_unselect_all(select);
        gtk_tree_selection_select_iter(select, &iter);
      }
      
      if (depth == 1) {
        gtk_tree_model_get(GTK_TREE_MODEL(ui_nap_get_net_tree()),
                           &iter, 0, &ng, -1);
        pop = ui_create_netgroup1_popup(ng);
      } else if (depth == 2) {
        gtk_tree_model_get(GTK_TREE_MODEL(ui_nap_get_net_tree()),
                           &iter, 0, &net, -1);
        pop = ui_create_network1_popup(net);
      } else if (depth == 3) {
        gtk_tree_model_get(GTK_TREE_MODEL(ui_nap_get_net_tree()),
                           &iter, 0, &server, -1);
        pop = ui_create_server1_popup(server);
      } else {
        g_warning("invalid depth in server tree");
      }
    }
  }
  if (pop) {
    gtk_menu_popup(GTK_MENU(pop), NULL, NULL, NULL, NULL,
                   event->button, event->time);
    return TRUE;
  } else {
    return FALSE;
  }
}

static void on_mode_activate(gpointer user_data) {
  int flag = GPOINTER_TO_INT(user_data);
  int mode = pref_get_ref("/nap/usermode", int);

  if (mode & flag) mode &= ~flag;
  else mode |= flag;
  pref_store("/nap/usermode", mode);
}

GtkWidget* ui_nap_mode_popup() {
  GtkWidget* popup;
  GtkWidget* mode;
  int cmode = pref_get_ref("/nap/usermode", int);
  int i1;
  GtkWidget* popup_on=NULL;
  GtkWidget* popup_sn=NULL;
  GtkWidget* popup_ng=NULL;

  popup = gtk_menu_new();

  for (i1 = 0; i1 < NAP_MODE_NUMBER; i1++) {
    int m = (1<<i1);
    if ((m & NAP_MODE_MASK_NAP)) {
      mode = gtk_check_menu_item_new_with_label(NapUserModes(i1));
      gtk_widget_show(mode);
      gtk_container_add(GTK_CONTAINER(popup), mode);
      if (cmode & m)
        gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mode), TRUE);
      g_signal_connect_swapped(G_OBJECT(mode), "activate",
                               G_CALLBACK(on_mode_activate), GINT_TO_POINTER(m));
    } else {
      if ((m & NAP_MODE_MASK_ON)) {
        if (!popup_on) popup_on = gtk_menu_new();
        mode = gtk_check_menu_item_new_with_label(NapUserModes(i1));
        gtk_widget_show(mode);
        gtk_container_add(GTK_CONTAINER(popup_on), mode);
        if (cmode & m)
          gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mode), TRUE);
        g_signal_connect_swapped(G_OBJECT(mode), "activate",
                                 G_CALLBACK(on_mode_activate), GINT_TO_POINTER(m));
      }
      if ((m & NAP_MODE_MASK_NG)) {
        if (!popup_ng) popup_ng = gtk_menu_new();
        mode = gtk_check_menu_item_new_with_label(NapUserModes(i1));
        gtk_widget_show(mode);
        gtk_container_add(GTK_CONTAINER(popup_ng), mode);
        if (cmode & m)
          gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mode), TRUE);
        g_signal_connect_swapped(G_OBJECT(mode), "activate",
                                 G_CALLBACK(on_mode_activate), GINT_TO_POINTER(m));
      }
      if ((m & NAP_MODE_MASK_SN)) {
        if (!popup_sn) popup_sn = gtk_menu_new();
        mode = gtk_check_menu_item_new_with_label(NapUserModes(i1));
        gtk_widget_show(mode);
        gtk_container_add(GTK_CONTAINER(popup_sn), mode);
        if (cmode & m)
          gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mode), TRUE);
        g_signal_connect_swapped(G_OBJECT(mode), "activate",
                                 G_CALLBACK(on_mode_activate), GINT_TO_POINTER(m));
      }
    }
  }

  if (popup_on) {
    gtk_widget_show(popup_on);
    mode = gtk_menu_item_new_with_label(_("Additional Opennap modes"));
    gtk_widget_show(mode);
    gtk_container_add(GTK_CONTAINER(popup), mode);
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(mode), popup_on);
  }
  if (popup_ng) {
    gtk_widget_show(popup_ng);
    mode = gtk_menu_item_new_with_label(_("Additional Opennap-ng modes"));
    gtk_widget_show(mode);
    gtk_container_add(GTK_CONTAINER(popup), mode);
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(mode), popup_ng);
  }
  if (popup_sn) {
    gtk_widget_show(popup_sn);
    mode = gtk_menu_item_new_with_label(_("Additional SlavaNap modes"));
    gtk_widget_show(mode);
    gtk_container_add(GTK_CONTAINER(popup), mode);
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(mode), popup_sn);
  }
  
  return popup;
}

static void on_service_remove(GtkComboBoxEntry* entry) {
  GtkListStore* store = ui_nap_get_service_store();
  GtkTreeIter iter;
  
  if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(entry), &iter)) {
    url_t* url;
    gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 0, &url, -1);
    object_kill(OBJ(url));
  }
}

static void on_service_changed(GtkComboBox* combo)
{
    int index = gtk_combo_box_get_active(combo);
    GtkWidget* entry = gtk_bin_get_child(GTK_BIN(combo));
    const char* text = ui_entry_get_text(GTK_ENTRY(entry));
    
    ui_event_label_set_sensitive(RefreshLabel, text && *text);
    gtk_widget_set_sensitive(RemoveButton, index != -1);
}

static GtkWidget* on_net_tip(void* object, unsigned depth) {
  GtkTable* table;
  int line = 0;
  nap_ng_t* ng;
  nap_net_t* net;
  nap_server_t* server;
  
  if (depth == 1) {
    ng = object;
    net = NULL;
    server = NULL;
  } else if (depth == 2) {
    net = object;
    ng = net->group;
    server = net->active;
  } else if (depth == 3) {
    server = object;
    net = server->net;
    ng = net->group;
  } else {
    g_assert(FALSE);
    return NULL;
  }

  table = GTK_TABLE(gtk_table_new(1, 2, FALSE));
  gtk_table_set_col_spacings(table, 10);
    
  if (ng) {
    ui_table_add_line(table, &line, _("Group"), OBJ(ng)->name);
    if (ng->keep < 0)
      ui_table_add_line(table, &line, _("Connect"), _("all networks"));
    else if (ng->keep == 1)
      ui_table_add_line(table, &line, _("Connect"), _("1 network"));
    else
      ui_table_add_line(table, &line, _("Connect"), l_printf(_("%d networks"), ng->keep));
  }
  if (net) {
    ui_table_add_sep(table, &line);
    ui_table_add_line(table, &line, _("Network"), OBJ(net)->name);
    ui_table_add_line(table, &line, _("Version"), nap_net_print_version(net));
    if (net->charset)
      ui_table_add_line(table, &line, _("Charset"), net->charset);
    if (server && server->status == NAP_SERVER_ONLINE) {
      ui_table_add_line(table, &line, _("Your nick"), net->cur_nick);
      ui_table_add_line(table, &line, _("Your level"), NapLevel(net->level));
      ui_table_add_line(table, &line, _("Active searches"), l_printf("%d", g_list_length(net->active_searches)));
      ui_table_add_line(table, &line, _("Queued searches"), l_printf("%d", g_list_length(net->queued_searches)));
    } else {
      ui_table_add_line(table, &line, _("Your nick"), nap_net_get_nick(net));
    }

    if (net->autojoin) {
      ui_table_add_line(table, &line, _("Autojoin"), make_string_from_list(net->autojoin, " "));
    }
  }
  if (server) {
    ui_table_add_sep(table, &line);
    if (server->ip)
      ui_table_add_line(table, &line, _("Server"), 
                        l_printf("%s (%s:%u)", OBJ(server)->name, ntoa(server->ip), server->port));
    else
      ui_table_add_line(table, &line, _("Server"), OBJ(server)->name);
    if (server->admin && *server->admin)
      ui_table_add_line(table, &line, _("Admin"), server->admin);
    if (server->web && *server->web)
      ui_table_add_line(table, &line, _("Website"), server->web);
  }

  gtk_widget_show(GTK_WIDGET(table));
  return GTK_WIDGET(table);

}

static void ui_nap_create_net_page(void) {
  GtkWidget* vbox;
  GtkWidget* hbox;
  GtkWidget* entry;
  GtkWidget* label;
  GtkWidget* button;
  GtkRequisition req;
  GtkWidget* scrolledwindow;
  GtkWidget* tv;
  GtkTreeViewColumn *column;
  GtkTreeSelection *select;
  GtkListStore* store;

  //
  vbox = gtk_vbox_new(FALSE, 5);
  gtk_widget_show(vbox);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);

  hbox = gtk_hbox_new(FALSE, 5);
  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

  label = gtk_label_new(_("<b>Server list service</b>"));
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
/*   label = ui_create_header_label(_("Server list service")); */
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 3);

  RemoveButton = button = ui_create_close_button(_("remove this service"));
  gtk_widget_show(button);
  gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);

  store = ui_nap_get_service_store();
  entry = gtk_combo_box_entry_new_with_model(GTK_TREE_MODEL(store), 1);
  gtk_widget_show(entry);
  gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
  gtk_widget_size_request(entry, &req);

  g_signal_connect_swapped(G_OBJECT(RemoveButton), "clicked",
                           G_CALLBACK(on_service_remove), entry);

  RefreshLabel = label = ui_create_event_label
    (_("get network list"), G_CALLBACK(on_server_refresh), entry, TRUE);
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

  if (gtk_combo_box_get_active(GTK_COMBO_BOX(entry)) == -1) {
    gtk_widget_set_sensitive(RemoveButton, FALSE);
    ui_event_label_set_sensitive(RefreshLabel, FALSE);
  }

  g_signal_connect(G_OBJECT(entry), "changed",
                   G_CALLBACK(on_service_changed), NULL);

  button =
    ui_create_popup_button(gtk_label_new(_("Usermode")), POS_RIGHT_DOWN,
                           (PopupFunction)ui_nap_mode_popup, NULL);
  gtk_widget_show(button);
  gtk_widget_set_size_request(button, -1, req.height);
  gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
  

  // NETWORK LIST
  scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow);
  gtk_box_pack_start(GTK_BOX(vbox), scrolledwindow, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow), 
                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow),
                                       GTK_SHADOW_IN);


  tv = gtk_tree_view_new_with_model
    (GTK_TREE_MODEL(ui_nap_get_net_tree()));
  //  gtk_tree_view_columns_autosize(GTK_TREE_VIEW(tv));
  gtk_container_add (GTK_CONTAINER (scrolledwindow), tv);
  gtk_widget_show(tv);
  gtk_tree_view_set_headers_clickable (GTK_TREE_VIEW (tv), TRUE);
  //  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tv), TRUE);
  ServerView = GTK_TREE_VIEW(tv);
  gtk_fancytips_new(GTK_TREE_VIEW(tv), (FancyTipsFunc)on_net_tip, 
                    GTK_FANCY_POS_MOUSE);

  select = gtk_tree_view_get_selection(GTK_TREE_VIEW(tv));
  gtk_tree_selection_set_mode(select, GTK_SELECTION_MULTIPLE);

  column = ui_tv_column_new
    (_("Name"), 
     gtk_cell_renderer_text_new(), "text", 1, TRUE,
     gtk_cell_renderer_pixbuf_new(), "pixbuf", 7, FALSE,
     gtk_cell_renderer_pixbuf_new(), "pixbuf", 8, FALSE,
     NULL);
  ui_tv_config_default_col(column, 1, "nap:net:col:name", FALSE);
  gtk_tree_view_append_column(GTK_TREE_VIEW (tv), column);
  
  column = ui_tv_column_new
    (_("Status"), gtk_cell_renderer_text_new(), "text", 3, TRUE, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW (tv), column);
  ui_tv_config_default_col(column, 3, "nap:net:col:status", FALSE);

  column = ui_tv_column_new
    (_("Failures"),
     gtk_cell_renderer_text_new(), "text", 9, FALSE, 
     gtk_cell_renderer_text_new(), "text", 10, TRUE,
     NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW (tv), column);
  ui_tv_config_default_col(column, 10, "nap:net:col:fails", FALSE);

  column = ui_tv_column_new
    (_("Users"), gtk_cell_renderer_text_new(), "text", 4, TRUE, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW (tv), column);
  ui_tv_config_default_col(column, 4, "nap:net:col:users", FALSE);

  column = ui_tv_column_new
    (_("Files"), gtk_cell_renderer_text_new(), "text", 5, TRUE, NULL);
  ui_tv_config_default_col(column, 5, "nap:net:col:files", FALSE);
  gtk_tree_view_append_column(GTK_TREE_VIEW (tv), column);

  column = ui_tv_column_new
    ("GiB", gtk_cell_renderer_text_new(), "text", 6, TRUE, NULL);
  ui_tv_config_default_col(column, 6, "nap:net:col:gigs", TRUE);
  gtk_tree_view_append_column(GTK_TREE_VIEW (tv), column);
  
  g_signal_connect(G_OBJECT (tv), "button-press-event",
                   G_CALLBACK(on_nap_server_tree_button),
                   NULL);

  ui_add_network_subpage(NAP_ID, vbox);
  
  ui_tv_config_line_marker(GTK_TREE_VIEW(tv), "network:entry:online", 2);

}

static void ui_nap_service_hide(url_t* serv)
{
    GtkTreeIter* iter = object_get_data(OBJ(serv), "NapServiceEntry");
    
    gtk_list_store_remove(ui_nap_get_service_store(), iter);
    object_free_data_common(OBJ(serv), "NapServiceEntry");
}

static void ui_nap_service_update(url_t* serv) {
    GtkTreeIter* iter = object_get_data(OBJ(serv), "NapServiceEntry");

    if (!iter) return;
    
    gtk_list_store_set(ui_nap_get_service_store(), iter,
                       0, serv,
                       1, l_printf("http://%s%s", serv->host, serv->doc),
                       -1);
}

static void ui_nap_service_show(url_t* serv) {
    GtkListStore* store = ui_nap_get_service_store();
    GtkTreeIter* iter = object_get_data(OBJ(serv), "NapServiceEntry");
  
    if (iter) return;
    object_set_data(OBJ(serv), "NapServerEntry", iter = g_malloc(sizeof(*iter)));
    gtk_list_store_append(store, iter);
    ui_nap_service_update(serv);
}

void ui_nap_service_new(url_t* serv)
{
    g_assert(serv);
    
    object_notify_d(OBJ(serv), (ObjectFunc)ui_nap_service_hide, NULL);
    object_monitor_d(OBJ(serv), (ObjectFunc)ui_nap_service_update, NULL);
    
    ui_nap_service_show(serv);
}
