/* Copyright (C) 2005 sgop@users.sourceforge.net This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.5 $
 * $Date: 2006/07/22 17:14:03 $
 * $Author: sid4u $
 */

#include "lcore/action_item.h"
#include "lcore/string_list.h"
#include "lcore/l_i18n.h"
#include "nap.h"
#include "nap_actions.h"
#include "nap_chat.h"
#include "nap_whois.h"
#include "nap_browse.h"
#include "nap_hotlist.h"
#include "nap_upload.h"
#include "nap_download.h"

static void on_nap_chat(GList* users) {
  GList* dlist;
  for (dlist = users; dlist; dlist = dlist->next) {
    nap_user_t* nu = dlist->data;
    if (nu->net) nap_cp_new_private(nu->net, OBJ(nu)->name);
  }
}

static void on_nap_whois(GList* users) {
  GList* dlist;
  for (dlist = users; dlist; dlist = dlist->next) {
    nap_user_t* nu = dlist->data;
    if (nu->net) nap_whois_user(nu, nu->net, NAP_WHOIS_WINDOW, FALSE);
  }
}

static gboolean on_nap_check_not_buddy(nap_user_t* nu) {
  return (!nap_hot_in_hotlist(nu));
}

static void on_nap_buddy(GList* users) {
  GList* dlist;
  for (dlist = users; dlist; dlist = dlist->next) {
    nap_user_t* nu = dlist->data;
    if (!nap_hot_in_hotlist(nu)) nap_hot_new(nu, NULL);
  }
}

static void on_nap_browse(GList* users) {
  GList* dlist;
  for (dlist = users; dlist; dlist = dlist->next) {
    nap_user_t* nu = dlist->data;
    if (nu->net) nap_browse_user(nu);
  }
}

static gboolean on_nap_check_mod(nap_user_t* nu) {
  return (nu->net && nu->net->level >= NAP_L_MOD);
}

static void on_nap_muzzle(GList* users) {
  GList* dlist;
  for (dlist = users; dlist; dlist = dlist->next) {
    nap_user_t* nu = dlist->data;
    if (nu->net) nap_user_muzzle(nu, nu->net, NULL);
  }
}

static void on_nap_kill(GList* users) {
  GList* dlist;
  for (dlist = users; dlist; dlist = dlist->next) {
    nap_user_t* nu = dlist->data;
    if (nu->net) nap_user_kill(nu, nu->net, NULL);
  }
}

static void on_nap_ban(GList* users) {
  GList* dlist;
  for (dlist = users; dlist; dlist = dlist->next) {
    nap_user_t* nu = dlist->data;
    if (nu->net) nap_user_ban(nu, nu->net, NULL, 0);
  }
}

static void on_nap_bankill(GList* users) {
  GList* dlist;
  for (dlist = users; dlist; dlist = dlist->next) {
    nap_user_t* nu = dlist->data;
    if (nu->net) {
      nap_user_ban(nu, nu->net, NULL, 0);
      nap_user_kill(nu, nu->net, NULL);
    }
  }
}

static void on_nap_nuke(GList* users) {
  GList* dlist;
  for (dlist = users; dlist; dlist = dlist->next) {
    nap_user_t* nu = dlist->data;
    if (nu->net) nap_user_nuke(nu, nu->net);
  }
}

static action_item_t* nap_actions_user_menu(const char* name) {
  action_item_t* item;

  item = action_item_root(name);
  action_item_simple(N_("Open chat..."), item, 0,
                     (ActionItemFunc)on_nap_chat);
  action_item_simple(N_("Send message..."), item, AI_RESTRICT_1,
                     NULL);
  action_item_sep(item);
  action_item_simple(N_("Whois user"), item, 0,
                     (ActionItemFunc)on_nap_whois);
  action_item_new(N_("Add to buddies"), item, 0,
                  (ActionItemFunc)on_nap_buddy,
                  (ActionItemCheck)on_nap_check_not_buddy);
  action_item_simple(N_("Browse user"), item, 0,
                     (ActionItemFunc)on_nap_browse);
  /*
  action_item_sep(item);
  action_item_new("Muzzle user on network", item, AI_HIDE_0,
                  (ActionItemFunc)on_nap_muzzle,
                  (ActionItemCheck)on_nap_check_mod);
  action_item_new("Kick user from network", item, AI_HIDE_0,
                  (ActionItemFunc)on_nap_kill,
                  (ActionItemCheck)on_nap_check_mod);
  action_item_new("Ban user from network", item, AI_HIDE_0,
                  (ActionItemFunc)on_nap_ban,
                  (ActionItemCheck)on_nap_check_mod);
  action_item_new("Ban/Kick user from network", item, AI_HIDE_0,
                  (ActionItemFunc)on_nap_bankill,
                  (ActionItemCheck)on_nap_check_mod);
  action_item_new("Nuke user account", item, AI_HIDE_0,
                  (ActionItemFunc)on_nap_nuke,
                  (ActionItemCheck)on_nap_check_mod);
  */
  return item;
}




static gboolean on_check_not_joined(nap_ce_t* ce) {
  return (!nap_cp_search_net(ce->net, ce->name, NAP_PAGE_PUBLIC) &&
	  NAP_NET_ONLINE(ce->net));
}
static gboolean on_check_joined(nap_ce_t* ce) {
  return (nap_cp_search_net(ce->net, ce->name, NAP_PAGE_PUBLIC) != NULL);
}

static void on_join_channel(GList* channels) {
  GList* dlist;
  for (dlist = channels; dlist; dlist = dlist->next) {
    nap_ce_t* ce = dlist->data;
    if (ce && !nap_cp_search_net(ce->net, ce->name, NAP_PAGE_PUBLIC))
      nap_join_channel(ce->net, ce->name);
  }
}

static void on_part_channel(GList* channels) {
  GList* dlist;
  for (dlist = channels; dlist; dlist = dlist->next) {
    nap_ce_t* ce = dlist->data;
    nap_cp_t* page = nap_cp_search_net(ce->net, ce->name, NAP_PAGE_PUBLIC);
    if (!page) continue;
    chat_page_close(PAGE(page));
  }
}

static gboolean on_check_not_auto(nap_ce_t* ce) {
  return (!string_slist_contains(ce->net->autojoin, ce->name));
}
static gboolean on_check_auto(nap_ce_t* ce) {
  return (string_slist_contains(ce->net->autojoin, ce->name) != NULL);
}

static void on_add_channel(GList* channels) {
  GList* dlist;
  for (dlist = channels; dlist; dlist = dlist->next) {
    nap_ce_t* ce = dlist->data;
    if (!string_slist_contains(ce->net->autojoin, ce->name)) {
      ce->net->autojoin =
        g_slist_append(ce->net->autojoin, g_strdup(ce->name));
    }
  }
}
static void on_remove_channel(GList* channels) {
  GList* dlist;
  for (dlist = channels; dlist; dlist = dlist->next) {
    nap_ce_t* ce = dlist->data;
    GSList* dl = string_slist_contains(ce->net->autojoin, ce->name);
    if (dl) {
      g_free(dl->data);
      ce->net->autojoin = g_slist_remove(ce->net->autojoin, dl->data);
    }
  }
}


/* static void on_refresh_channels(GList* dummy, nap_cl_t* ncl) { */
/*   (void)dummy; */
/*   if (ncl) nap_get_channel_list(ncl); */
/* } */

static void nap_actions_channel_list() {
  action_item_t* item;
  //  action_item_t* sub;

  item = action_item_root("Nap channel list actions");
  action_item_new(N_("Join channel"), item, 0,
                  (ActionItemFunc)on_join_channel,
                  (ActionItemCheck)on_check_not_joined);
  action_item_new(N_("Part channel"), item, 0,
                  (ActionItemFunc)on_part_channel,
                  (ActionItemCheck)on_check_joined);
  action_item_sep(item);
  action_item_new(N_("Add to autojoin"), item, AI_HIDE_0,
                  (ActionItemFunc)on_add_channel,
                  (ActionItemCheck)on_check_not_auto);
  action_item_new(N_("Remove from autojoin"), item, AI_HIDE_0,
                  (ActionItemFunc)on_remove_channel,
                  (ActionItemCheck)on_check_auto);
  //FIXME
/*   action_item_sep(item); */
/*   action_item_simple("Refresh list", item, AI_GLOBAL, */
/*                      (ActionItemFunc)on_refresh_channels); */
}



static nap_user_t* nap_cu_to_nu(nap_cu_t* cu, nap_cp_t* page) {
  nap_user_t* user =
    nap_user_find(OBJ(cu)->name, page?page->net:NULL);
  return user;
}

static nap_user_t* nap_upload_to_nu(nap_upload_t* upload) {
  return NAP_USER(UPLOAD(upload)->user);
}

static nap_user_t* nap_source_to_nu(nap_source_t* source) {
  return NAP_USER(SOURCE(source)->user);
}

static nap_user_t* nap_result_to_nu(search_result_t* result) {
  return NAP_USER(result->user);
}

static nap_user_t* nap_browse_to_nu(file_t* file, browse_t* browse) {
  (void)file;
  return NAP_USER(browse->user);
}

void nap_actions_init() {
  action_item_t* item;

  item = nap_actions_user_menu("Online nap users actions");
  item->convert = (ActionObjectConvert)nap_cu_to_nu;

  item = nap_actions_user_menu("Upload nap users actions");
  item->convert = (ActionObjectConvert)nap_upload_to_nu;
  user_action_upload_register(NAP_ID, item);

  item = nap_actions_user_menu("Source nap users actions");
  item->convert = (ActionObjectConvert)nap_source_to_nu;
  user_action_source_register(NAP_ID, item);

  item = nap_actions_user_menu("Result nap users actions");
  item->convert = (ActionObjectConvert)nap_result_to_nu;
  user_action_result_register(NAP_ID, item);

  item = nap_actions_user_menu("Browse nap users actions");
  item->convert = (ActionObjectConvert)nap_browse_to_nu;
  user_action_browse_register(NAP_ID, item);

  nap_actions_channel_list();
}
