/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.6 $
 * $Date: 2006/07/22 17:14:03 $
 * $Author: sid4u $
 */

#include "lcore/plugin.h"
#include "lcore/ui.h"
#include "lcore/dynamic.h"
#include "nap_ui.h"

static ui_nap_t NAPUI;

typedef struct {
    module_t _module;
    void (*init)(ui_nap_t*);
    void (*run)(void);
} nap_ui_module_t;

static nap_ui_module_t* NapUIModule = NULL;

#define CHECK_SYMBOL(sym)                                               \
  if (!(sym)) {                                                         \
    g_log("NAP", G_LOG_LEVEL_WARNING, "could not find symbol '%s'", #sym); \
  }

void nap_ui_load_syms()
{
    const char* filename;
    GModule* handle;
    
    filename = plugin_find_ui_library("nap");
    if (!filename)
    {
        g_log("NAP", G_LOG_LEVEL_DEBUG, "starting plugin without interface");
        return;
    }
    
    handle = lopster_dll_open(filename, G_MODULE_BIND_LAZY|G_MODULE_BIND_LOCAL);
    if (!handle)
    {
        g_log("NAP", G_LOG_LEVEL_WARNING, "%s", g_module_error());
        g_log("NAP", G_LOG_LEVEL_DEBUG, "starting the plugin without interface");
        return;
    }
    
    CHECK_SYMBOL(g_module_symbol(handle, "ui_nap_init", (void**)&NapUIModule->init));
    CHECK_SYMBOL(g_module_symbol(handle, "ui_nap_run", (void**)&NapUIModule->run));
}


void NAPUI_channel_joined(nap_pub_t* page) {
  if (NAPUI.channel_joined) NAPUI.channel_joined(page);
}
void NAPUI_cp_update_op(nap_pub_t* page) {
  if (NAPUI.cp_update_op) NAPUI.cp_update_op(page);
}
void NAPUI_new_topic(nap_pub_t* page) {
  if (NAPUI.new_topic) NAPUI.new_topic(page);
}
void NAPUI_user_joined(nap_pub_t* page, nap_cu_t* user) {
  if (NAPUI.user_joined) NAPUI.user_joined(page, user);
}
void NAPUI_user_parted(nap_pub_t* page, nap_cu_t* user) {
  if (NAPUI.user_parted) NAPUI.user_parted(page, user);
}
void NAPUI_whois_show(nap_user_t* user) {
  if (NAPUI.whois_show) NAPUI.whois_show(user);
}



static void nap_ui_init() {
  NapUIModule->init(&NAPUI);
}

static void nap_ui_run() {
  NapUIModule->run();
}

static void nap_ui_module_init(nap_ui_module_t* module)
{
    module_init(MODULE(module), "Nap User Interface Module",
                (ModuleFunc)nap_ui_init, (ModuleFunc)nap_ui_run, NULL);
    
    module->init = NULL;
    module->run = NULL;
    
    module_require(ui_module_instance);
    module_require(dynamic_module_instance);
    
    nap_ui_load_syms();
}

module_t* nap_ui_module_instance()
{
    if (!NapUIModule)
    {
        NapUIModule = g_malloc0(sizeof(*NapUIModule));
        nap_ui_module_init(NapUIModule);
    }
    return MODULE(NapUIModule);
}

#undef CHECK_SYMBOL
